/**
  *******************************************************************************************************
  * @file    fm33fr0xx_fl_cmu.h
  * @author  FMSH Application Team
  * @brief   Head file of CMU FL Module
  *******************************************************************************************************
  * @attention
  *
  * Copyright (c) [2019] [Fudan Microelectronics]
  * THIS SOFTWARE is licensed under the Mulan PSL v1.
  * can use this software according to the terms and conditions of the Mulan PSL v1.
  * You may obtain a copy of Mulan PSL v1 at:
  * http://license.coscl.org.cn/MulanPSL
  * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND, EITHER EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT, MERCHANTABILITY OR FIT FOR A PARTICULAR
  * PURPOSE.
  * See the Mulan PSL v1 for more details.
  *
  *******************************************************************************************************
  */


/* Define to prevent recursive inclusion---------------------------------------------------------------*/
#ifndef __FM33FR0XX_FL_CMU_H
#define __FM33FR0XX_FL_CMU_H

#ifdef __cplusplus
extern "C" {
#endif
/* Includes -------------------------------------------------------------------------------------------*/
#include "fm33fr0xx_fl_def.h"
/** @addtogroup FM33FR0XX_FL_Driver
  * @{
  */
/* Exported types -------------------------------------------------------------------------------------*/
/** @defgroup CMU_FL_ES_INIT CMU Exported Init structures
  * @{
  */

/**
  * @brief FL CMU Init Sturcture definition
  */
typedef enum
{
    FL_SYSTEM_CLOCK_RCHF_8M = 0,
    FL_SYSTEM_CLOCK_RCHF_16M,
    FL_SYSTEM_CLOCK_RCHF_24M,
    FL_SYSTEM_CLOCK_PLL_RCHF_32M,
    FL_SYSTEM_CLOCK_PLL_RCHF_48M,
    FL_SYSTEM_CLOCK_RCLP,
    FL_SYSTEM_CLOCK_XTLF,
    FL_SYSTEM_CLOCK_XTHF,
} FL_SystemClock;
/**
  * @}
  */
/* Exported constants ---------------------------------------------------------------------------------*/
/** @defgroup CMU_FL_Exported_Constants CMU Exported Constants
  * @{
  */

#define    CMU_SCCR_LSCATS_Pos                                    (27U)
#define    CMU_SCCR_LSCATS_Msk                                    (0x1U << CMU_SCCR_LSCATS_Pos)
#define    CMU_SCCR_LSCATS                                        CMU_SCCR_LSCATS_Msk

#define    CMU_SCCR_SLP_ENEXTI_Pos                                (25U)
#define    CMU_SCCR_SLP_ENEXTI_Msk                                (0x1U << CMU_SCCR_SLP_ENEXTI_Pos)
#define    CMU_SCCR_SLP_ENEXTI                                    CMU_SCCR_SLP_ENEXTI_Msk

#define    CMU_SCCR_APBPRES_Pos                                   (16U)
#define    CMU_SCCR_APBPRES_Msk                                   (0x7U << CMU_SCCR_APBPRES_Pos)
#define    CMU_SCCR_APBPRES                                       CMU_SCCR_APBPRES_Msk

#define    CMU_SCCR_AHBPRES_Pos                                   (8U)
#define    CMU_SCCR_AHBPRES_Msk                                   (0x7U << CMU_SCCR_AHBPRES_Pos)
#define    CMU_SCCR_AHBPRES                                       CMU_SCCR_AHBPRES_Msk

#define    CMU_SCCR_STCLKSEL_Pos                                  (6U)
#define    CMU_SCCR_STCLKSEL_Msk                                  (0x3U << CMU_SCCR_STCLKSEL_Pos)
#define    CMU_SCCR_STCLKSEL                                      CMU_SCCR_STCLKSEL_Msk

#define    CMU_SCCR_SYSCLKSEL_Pos                                 (0U)
#define    CMU_SCCR_SYSCLKSEL_Msk                                 (0x7U << CMU_SCCR_SYSCLKSEL_Pos)
#define    CMU_SCCR_SYSCLKSEL                                     CMU_SCCR_SYSCLKSEL_Msk

#define    CMU_RCHFCR_FSEL_Pos                                    (16U)
#define    CMU_RCHFCR_FSEL_Msk                                    (0xfU << CMU_RCHFCR_FSEL_Pos)
#define    CMU_RCHFCR_FSEL                                        CMU_RCHFCR_FSEL_Msk

#define    CMU_RCHFCR_EN_Pos                                      (0U)
#define    CMU_RCHFCR_EN_Msk                                      (0x1U << CMU_RCHFCR_EN_Pos)
#define    CMU_RCHFCR_EN                                          CMU_RCHFCR_EN_Msk

#define    CMU_RCHFTR_TRIM_Pos                                    (0U)
#define    CMU_RCHFTR_TRIM_Msk                                    (0xffU << CMU_RCHFTR_TRIM_Pos)
#define    CMU_RCHFTR_TRIM                                        CMU_RCHFTR_TRIM_Msk

#define    CMU_PLLCR_DB_Pos                                       (16U)
#define    CMU_PLLCR_DB_Msk                                       (0x3fU << CMU_PLLCR_DB_Pos)
#define    CMU_PLLCR_DB                                           CMU_PLLCR_DB_Msk

#define    CMU_PLLCR_LOCKED_Pos                                   (7U)
#define    CMU_PLLCR_LOCKED_Msk                                   (0x1U << CMU_PLLCR_LOCKED_Pos)
#define    CMU_PLLCR_LOCKED                                       CMU_PLLCR_LOCKED_Msk

#define    CMU_PLLCR_REFPRSC_Pos                                  (4U)
#define    CMU_PLLCR_REFPRSC_Msk                                  (0x7U << CMU_PLLCR_REFPRSC_Pos)
#define    CMU_PLLCR_REFPRSC                                      CMU_PLLCR_REFPRSC_Msk

#define    CMU_PLLCR_OSEL_Pos                                     (3U)
#define    CMU_PLLCR_OSEL_Msk                                     (0x1U << CMU_PLLCR_OSEL_Pos)
#define    CMU_PLLCR_OSEL                                         CMU_PLLCR_OSEL_Msk

#define    CMU_PLLCR_INSEL_Pos                                    (1U)
#define    CMU_PLLCR_INSEL_Msk                                    (0x1U << CMU_PLLCR_INSEL_Pos)
#define    CMU_PLLCR_INSEL                                        CMU_PLLCR_INSEL_Msk

#define    CMU_PLLCR_EN_Pos                                       (0U)
#define    CMU_PLLCR_EN_Msk                                       (0x1U << CMU_PLLCR_EN_Pos)
#define    CMU_PLLCR_EN                                           CMU_PLLCR_EN_Msk

#define    CMU_RCLPCR_ENB_Pos                                     (0U)
#define    CMU_RCLPCR_ENB_Msk                                     (0x1U << CMU_RCLPCR_ENB_Pos)
#define    CMU_RCLPCR_ENB                                         CMU_RCLPCR_ENB_Msk

#define    CMU_RCLPTR_TRIM_Pos                                    (0U)
#define    CMU_RCLPTR_TRIM_Msk                                    (0xffU << CMU_RCLPTR_TRIM_Pos)
#define    CMU_RCLPTR_TRIM                                        CMU_RCLPTR_TRIM_Msk

#define    CMU_XTLFCR_WAIT_Pos                                    (13U)
#define    CMU_XTLFCR_WAIT_Msk                                    (0x7U << CMU_XTLFCR_WAIT_Pos)
#define    CMU_XTLFCR_WAIT                                        CMU_XTLFCR_WAIT_Msk

#define    CMU_XTLFCR_EXCKSEN_Pos                                 (12U)
#define    CMU_XTLFCR_EXCKSEN_Msk                                 (0x1U << CMU_XTLFCR_EXCKSEN_Pos)
#define    CMU_XTLFCR_EXCKSEN                                     CMU_XTLFCR_EXCKSEN_Msk

#define    CMU_XTLFCR_EN_Pos                                      (8U)
#define    CMU_XTLFCR_EN_Msk                                      (0xfU << CMU_XTLFCR_EN_Pos)
#define    CMU_XTLFCR_EN                                          CMU_XTLFCR_EN_Msk

#define    CMU_XTLFCR_RDY_Pos                                     (4U)
#define    CMU_XTLFCR_RDY_Msk                                     (0x1U << CMU_XTLFCR_RDY_Pos)
#define    CMU_XTLFCR_RDY                                         CMU_XTLFCR_RDY_Msk

#define    CMU_XTLFCR_IPW_Pos                                     (0U)
#define    CMU_XTLFCR_IPW_Msk                                     (0xfU << CMU_XTLFCR_IPW_Pos)
#define    CMU_XTLFCR_IPW                                         CMU_XTLFCR_IPW_Msk

#define    CMU_LSSR_LSSTA_Pos                                     (8U)
#define    CMU_LSSR_LSSTA_Msk                                     (0x1U << CMU_LSSR_LSSTA_Pos)
#define    CMU_LSSR_LSSTA                                         CMU_LSSR_LSSTA_Msk

#define    CMU_LSSR_SEL_Pos                                       (0U)
#define    CMU_LSSR_SEL_Msk                                       (0xffU << CMU_LSSR_SEL_Pos)
#define    CMU_LSSR_SEL                                           CMU_LSSR_SEL_Msk

#define    CMU_XTHFCR_CAPCFG_Pos                                  (16U)
#define    CMU_XTHFCR_CAPCFG_Msk                                  (0x7U << CMU_XTHFCR_CAPCFG_Pos)
#define    CMU_XTHFCR_CAPCFG                                      CMU_XTHFCR_CAPCFG_Msk

#define    CMU_XTHFCR_WAIT_Pos                                    (13U)
#define    CMU_XTHFCR_WAIT_Msk                                    (0x7U << CMU_XTHFCR_WAIT_Pos)
#define    CMU_XTHFCR_WAIT                                        CMU_XTHFCR_WAIT_Msk

#define    CMU_XTHFCR_CFG_Pos                                     (8U)
#define    CMU_XTHFCR_CFG_Msk                                     (0x1fU << CMU_XTHFCR_CFG_Pos)
#define    CMU_XTHFCR_CFG                                         CMU_XTHFCR_CFG_Msk

#define    CMU_XTHFCR_RDY_Pos                                     (1U)
#define    CMU_XTHFCR_RDY_Msk                                     (0x1U << CMU_XTHFCR_RDY_Pos)
#define    CMU_XTHFCR_RDY                                         CMU_XTHFCR_RDY_Msk

#define    CMU_XTHFCR_EN_Pos                                      (0U)
#define    CMU_XTHFCR_EN_Msk                                      (0x1U << CMU_XTHFCR_EN_Pos)
#define    CMU_XTHFCR_EN                                          CMU_XTHFCR_EN_Msk

#define    CMU_IER_SYSCKE_IE_Pos                                  (2U)
#define    CMU_IER_SYSCKE_IE_Msk                                  (0x1U << CMU_IER_SYSCKE_IE_Pos)
#define    CMU_IER_SYSCKE_IE                                      CMU_IER_SYSCKE_IE_Msk

#define    CMU_IER_HFDET_IE_Pos                                   (1U)
#define    CMU_IER_HFDET_IE_Msk                                   (0x1U << CMU_IER_HFDET_IE_Pos)
#define    CMU_IER_HFDET_IE                                       CMU_IER_HFDET_IE_Msk

#define    CMU_IER_LFDET_IE_Pos                                   (0U)
#define    CMU_IER_LFDET_IE_Msk                                   (0x1U << CMU_IER_LFDET_IE_Pos)
#define    CMU_IER_LFDET_IE                                       CMU_IER_LFDET_IE_Msk

#define    CMU_ISR_HFDETO_Pos                                     (9U)
#define    CMU_ISR_HFDETO_Msk                                     (0x1U << CMU_ISR_HFDETO_Pos)
#define    CMU_ISR_HFDETO                                         CMU_ISR_HFDETO_Msk

#define    CMU_ISR_LFDETO_Pos                                     (8U)
#define    CMU_ISR_LFDETO_Msk                                     (0x1U << CMU_ISR_LFDETO_Pos)
#define    CMU_ISR_LFDETO                                         CMU_ISR_LFDETO_Msk

#define    CMU_ISR_SYSCSE_IF_Pos                                  (2U)
#define    CMU_ISR_SYSCSE_IF_Msk                                  (0x1U << CMU_ISR_SYSCSE_IF_Pos)
#define    CMU_ISR_SYSCSE_IF                                      CMU_ISR_SYSCSE_IF_Msk

#define    CMU_ISR_HFDET_IF_Pos                                   (1U)
#define    CMU_ISR_HFDET_IF_Msk                                   (0x1U << CMU_ISR_HFDET_IF_Pos)
#define    CMU_ISR_HFDET_IF                                       CMU_ISR_HFDET_IF_Msk

#define    CMU_ISR_LFDET_IF_Pos                                   (0U)
#define    CMU_ISR_LFDET_IF_Msk                                   (0x1U << CMU_ISR_LFDET_IF_Pos)
#define    CMU_ISR_LFDET_IF                                       CMU_ISR_LFDET_IF_Msk

#define    CMU_OPCCR1_EXTICKS_Pos                                 (30U)
#define    CMU_OPCCR1_EXTICKS_Msk                                 (0x1U << CMU_OPCCR1_EXTICKS_Pos)
#define    CMU_OPCCR1_EXTICKS                                     CMU_OPCCR1_EXTICKS_Msk

#define    CMU_OPCCR1_LPUARTCKS_Pos                               (24U)
#define    CMU_OPCCR1_LPUARTCKS_Msk                               (0x3U << CMU_OPCCR1_LPUARTCKS_Pos)
#define    CMU_OPCCR1_LPUARTCKS                                   CMU_OPCCR1_LPUARTCKS_Msk

#define    CMU_OPCCR1_I2CSMB1_Pos                                 (18U)
#define    CMU_OPCCR1_I2CSMB1_Msk                                 (0x3U << CMU_OPCCR1_I2CSMB1_Pos)
#define    CMU_OPCCR1_I2CSMB1                                     CMU_OPCCR1_I2CSMB1_Msk

#define    CMU_OPCCR1_I2CSMB0_Pos                                 (16U)
#define    CMU_OPCCR1_I2CSMB0_Msk                                 (0x3U << CMU_OPCCR1_I2CSMB0_Pos)
#define    CMU_OPCCR1_I2CSMB0                                     CMU_OPCCR1_I2CSMB0_Msk

#define    CMU_OPCCR1_BT16CKS_Pos                                 (14U)
#define    CMU_OPCCR1_BT16CKS_Msk                                 (0x3U << CMU_OPCCR1_BT16CKS_Pos)
#define    CMU_OPCCR1_BT16CKS                                     CMU_OPCCR1_BT16CKS_Msk

#define    CMU_OPCCR1_LPT16CKS_Pos                                (10U)
#define    CMU_OPCCR1_LPT16CKS_Msk                                (0x3U << CMU_OPCCR1_LPT16CKS_Pos)
#define    CMU_OPCCR1_LPT16CKS                                    CMU_OPCCR1_LPT16CKS_Msk

#define    CMU_OPCCR1_CAN1CKS_Pos                                 (8U)
#define    CMU_OPCCR1_CAN1CKS_Msk                                 (0x3U << CMU_OPCCR1_CAN1CKS_Pos)
#define    CMU_OPCCR1_CAN1CKS                                     CMU_OPCCR1_CAN1CKS_Msk

#define    CMU_OPCCR1_ATCKS_Pos                                   (7U)
#define    CMU_OPCCR1_ATCKS_Msk                                   (0x1U << CMU_OPCCR1_ATCKS_Pos)
#define    CMU_OPCCR1_ATCKS                                       CMU_OPCCR1_ATCKS_Msk

#define    CMU_OPCCR1_CAN0CKS_Pos                                 (4U)
#define    CMU_OPCCR1_CAN0CKS_Msk                                 (0x3U << CMU_OPCCR1_CAN0CKS_Pos)
#define    CMU_OPCCR1_CAN0CKS                                     CMU_OPCCR1_CAN0CKS_Msk

#define    CMU_OPCCR1_UART1CKS_Pos                                (2U)
#define    CMU_OPCCR1_UART1CKS_Msk                                (0x3U << CMU_OPCCR1_UART1CKS_Pos)
#define    CMU_OPCCR1_UART1CKS                                    CMU_OPCCR1_UART1CKS_Msk

#define    CMU_OPCCR1_UART0CKS_Pos                                (0U)
#define    CMU_OPCCR1_UART0CKS_Msk                                (0x3U << CMU_OPCCR1_UART0CKS_Pos)
#define    CMU_OPCCR1_UART0CKS                                    CMU_OPCCR1_UART0CKS_Msk

#define    CMU_OPCCR2_RTCPRSC_Pos                                 (22U)
#define    CMU_OPCCR2_RTCPRSC_Msk                                 (0x3ffU << CMU_OPCCR2_RTCPRSC_Pos)
#define    CMU_OPCCR2_RTCPRSC                                     CMU_OPCCR2_RTCPRSC_Msk

#define    CMU_OPCCR2_RTCCKS_Pos                                  (16U)
#define    CMU_OPCCR2_RTCCKS_Msk                                  (0x1U << CMU_OPCCR2_RTCCKS_Pos)
#define    CMU_OPCCR2_RTCCKS                                      CMU_OPCCR2_RTCCKS_Msk

#define    CMU_OPCCR2_RNGPRSC_Pos                                 (10U)
#define    CMU_OPCCR2_RNGPRSC_Msk                                 (0x7U << CMU_OPCCR2_RNGPRSC_Pos)
#define    CMU_OPCCR2_RNGPRSC                                     CMU_OPCCR2_RNGPRSC_Msk

#define    CMU_OPCCR2_ADCPRSC_Pos                                 (2U)
#define    CMU_OPCCR2_ADCPRSC_Msk                                 (0x7U << CMU_OPCCR2_ADCPRSC_Pos)
#define    CMU_OPCCR2_ADCPRSC                                     CMU_OPCCR2_ADCPRSC_Msk

#define    CMU_OPCCR2_ADCCKS_Pos                                  (0U)
#define    CMU_OPCCR2_ADCCKS_Msk                                  (0x3U << CMU_OPCCR2_ADCCKS_Pos)
#define    CMU_OPCCR2_ADCCKS                                      CMU_OPCCR2_ADCCKS_Msk

#define    CMU_AMCR_MPRIL_Pos                                     (0U)
#define    CMU_AMCR_MPRIL_Msk                                     (0x1U << CMU_AMCR_MPRIL_Pos)
#define    CMU_AMCR_MPRIL                                         CMU_AMCR_MPRIL_Msk

#define    CMU_CCCR_CCLIE_Pos                                     (1U)
#define    CMU_CCCR_CCLIE_Msk                                     (0x1U << CMU_CCCR_CCLIE_Pos)
#define    CMU_CCCR_CCLIE                                         CMU_CCCR_CCLIE_Msk

#define    CMU_CCCR_EN_Pos                                        (0U)
#define    CMU_CCCR_EN_Msk                                        (0x1U << CMU_CCCR_EN_Pos)
#define    CMU_CCCR_EN                                            CMU_CCCR_EN_Msk

#define    CMU_CCFR_CALPSC_Pos                                    (10U)
#define    CMU_CCFR_CALPSC_Msk                                    (0x3U << CMU_CCFR_CALPSC_Pos)
#define    CMU_CCFR_CALPSC                                        CMU_CCFR_CALPSC_Msk

#define    CMU_CCFR_REFPSC_Pos                                    (8U)
#define    CMU_CCFR_REFPSC_Msk                                    (0x3U << CMU_CCFR_REFPSC_Pos)
#define    CMU_CCFR_REFPSC                                        CMU_CCFR_REFPSC_Msk

#define    CMU_CCFR_CALSEL_Pos                                    (1U)
#define    CMU_CCFR_CALSEL_Msk                                    (0x3U << CMU_CCFR_CALSEL_Pos)
#define    CMU_CCFR_CALSEL                                        CMU_CCFR_CALSEL_Msk

#define    CMU_CCFR_REFSEL_Pos                                    (0U)
#define    CMU_CCFR_REFSEL_Msk                                    (0x1U << CMU_CCFR_REFSEL_Pos)
#define    CMU_CCFR_REFSEL                                        CMU_CCFR_REFSEL_Msk

#define    CMU_CCNR_CCNT_Pos                                      (0U)
#define    CMU_CCNR_CCNT_Msk                                      (0xffffU << CMU_CCNR_CCNT_Pos)
#define    CMU_CCNR_CCNT                                          CMU_CCNR_CCNT_Msk

#define    CMU_CCISR_CCLIF_Pos                                    (0U)
#define    CMU_CCISR_CCLIF_Msk                                    (0x1U << CMU_CCISR_CCLIF_Pos)
#define    CMU_CCISR_CCLIF                                        CMU_CCISR_CCLIF_Msk



#define    FL_CMU_GROUP1_BUSCLK_LPTIM16                           (0x1U << 1U)
#define    FL_CMU_GROUP1_BUSCLK_RTC                              (0x1U << 2U)
#define    FL_CMU_GROUP1_BUSCLK_PMU                               (0x1U << 3U)
#define    FL_CMU_GROUP1_BUSCLK_SCU                               (0x1U << 4U)
#define    FL_CMU_GROUP1_BUSCLK_IWDT                              (0x1U << 5U)
#define    FL_CMU_GROUP1_BUSCLK_ANAC                              (0x1U << 6U)
#define    FL_CMU_GROUP1_BUSCLK_PAD                               (0x1U << 7U)
#define    FL_CMU_GROUP1_BUSCLK_SVD                               (0x1U << 8U)
#define    FL_CMU_GROUP1_BUSCLK_COMP                              (0x1U << 9U)
#define    FL_CMU_GROUP2_BUSCLK_CRC                               (0x1U << 0U)
#define    FL_CMU_GROUP2_BUSCLK_RNG                               (0x1U << 1U)
#define    FL_CMU_GROUP2_BUSCLK_AES                               (0x1U << 2U)
#define    FL_CMU_GROUP2_BUSCLK_LCD                               (0x1U << 3U)
#define    FL_CMU_GROUP2_BUSCLK_DMA                               (0x1U << 4U)
#define    FL_CMU_GROUP2_BUSCLK_FLASH                             (0x1U << 5U)
#define    FL_CMU_GROUP2_BUSCLK_RAMBIST                           (0x1U << 6U)
#define    FL_CMU_GROUP2_BUSCLK_WWDT                              (0x1U << 7U)
#define    FL_CMU_GROUP2_BUSCLK_ADC                               (0x1U << 8U)
#define    FL_CMU_GROUP2_BUSCLK_DIVAS                             (0x1U << 9U)
#define    FL_CMU_GROUP2_BUSCLK_TSI                               (0x1U << 10U)
#define    FL_CMU_GROUP2_BUSCLK_PGL                               (0x1U << 11U)
#define    FL_CMU_GROUP2_BUSCLK_INTMUX                            (0x1U << 12U)
#define    FL_CMU_GROUP3_BUSCLK_SPI0                              (0x1U << 0U)
#define    FL_CMU_GROUP3_BUSCLK_SPI1                              (0x1U << 1U)
#define    FL_CMU_GROUP3_BUSCLK_SPI2                              (0x1U << 2U)
#define    FL_CMU_GROUP3_BUSCLK_SPI3                              (0x1U << 3U)
#define    FL_CMU_GROUP3_BUSCLK_UART0                             (0x1U << 8U)
#define    FL_CMU_GROUP3_BUSCLK_UART1                             (0x1U << 9U)
#define    FL_CMU_GROUP3_BUSCLK_UART2                             (0x1U << 10U)
#define    FL_CMU_GROUP3_BUSCLK_UART3                             (0x1U << 11U)
#define    FL_CMU_GROUP3_BUSCLK_UART4                             (0x1U << 12U)
#define    FL_CMU_GROUP3_BUSCLK_UART5                             (0x1U << 13U)
#define    FL_CMU_GROUP3_BUSCLK_UARTIR                            (0x1U << 14U)
#define    FL_CMU_GROUP3_BUSCLK_LPUART                            (0x1U << 16U)
#define    FL_CMU_GROUP3_BUSCLK_CAN0                              (0x1U << 18U)
#define    FL_CMU_GROUP3_BUSCLK_CAN1                              (0x1U << 19U)
#define    FL_CMU_GROUP3_BUSCLK_I2C0                              (0x1U << 24U)
#define    FL_CMU_GROUP3_BUSCLK_I2C1                              (0x1U << 25U)
#define    FL_CMU_GROUP3_BUSCLK_I2C2                              (0x1U << 26U)
#define    FL_CMU_GROUP3_BUSCLK_I2C3                              (0x1U << 27U)
#define    FL_CMU_GROUP3_BUSCLK_I2CSMB0                           (0x1U << 28U)
#define    FL_CMU_GROUP3_BUSCLK_I2CSMB1                           (0x1U << 29U)
#define    FL_CMU_GROUP4_BUSCLK_ATIM                              (0x1U << 0U)
#define    FL_CMU_GROUP4_BUSCLK_TAU0                              (0x1U << 4U)
#define    FL_CMU_GROUP4_BUSCLK_TAU1                              (0x1U << 5U)
#define    FL_CMU_GROUP4_BUSCLK_BSTIM16                           (0x1U << 8U)
#define    FL_CMU_OPCLK_EXTI                                      (0x1U << 31U)
#define    FL_CMU_OPCLK_FLASH                                     (0x1U << 30U)
#define    FL_CMU_OPCLK_LPUART                                    (0x1U << 28U)
#define    FL_CMU_OPCLK_RNG                                       (0x1U << 24U)
#define    FL_CMU_OPCLK_I2CSMB1                                   (0x1U << 21U)
#define    FL_CMU_OPCLK_I2CSMB0                                   (0x1U << 20U)
#define    FL_CMU_OPCLK_ADC                                       (0x1U << 16U)
#define    FL_CMU_OPCLK_ATIM                                      (0x1U << 15U)
#define    FL_CMU_OPCLK_CAN1                                      (0x1U << 14U)
#define    FL_CMU_OPCLK_CAN0                                      (0x1U << 13U)
#define    FL_CMU_OPCLK_LIN                                       (0x1U << 10U)
#define    FL_CMU_OPCLK_UART1                                     (0x1U << 9U)
#define    FL_CMU_OPCLK_UART0                                     (0x1U << 8U)
#define    FL_CMU_OPCLK_BSTIM16                                   (0x1U << 3U)
#define    FL_CMU_OPCLK_LPTIM16                                   (0x1U << 1U)



#define    FL_CMU_APBCLK_PSC_DIV1                                 (0x0U << CMU_SCCR_APBPRES_Pos)
#define    FL_CMU_APBCLK_PSC_DIV2                                 (0x4U << CMU_SCCR_APBPRES_Pos)
#define    FL_CMU_APBCLK_PSC_DIV4                                 (0x5U << CMU_SCCR_APBPRES_Pos)
#define    FL_CMU_APBCLK_PSC_DIV8                                 (0x6U << CMU_SCCR_APBPRES_Pos)
#define    FL_CMU_APBCLK_PSC_DIV16                                (0x7U << CMU_SCCR_APBPRES_Pos)


#define    FL_CMU_AHBCLK_PSC_DIV1                                 (0x0U << CMU_SCCR_AHBPRES_Pos)
#define    FL_CMU_AHBCLK_PSC_DIV2                                 (0x4U << CMU_SCCR_AHBPRES_Pos)
#define    FL_CMU_AHBCLK_PSC_DIV4                                 (0x5U << CMU_SCCR_AHBPRES_Pos)
#define    FL_CMU_AHBCLK_PSC_DIV8                                 (0x6U << CMU_SCCR_AHBPRES_Pos)
#define    FL_CMU_AHBCLK_PSC_DIV16                                (0x7U << CMU_SCCR_AHBPRES_Pos)


#define    FL_CMU_SYSTICK_CLK_SOURCE_SCLK                         (0x0U << CMU_SCCR_STCLKSEL_Pos)
#define    FL_CMU_SYSTICK_CLK_SOURCE_LSCLK                        (0x1U << CMU_SCCR_STCLKSEL_Pos)
#define    FL_CMU_SYSTICK_CLK_SOURCE_SYSCLK                       (0x3U << CMU_SCCR_STCLKSEL_Pos)


#define    FL_CMU_SYSTEM_CLK_SOURCE_RCHF                          (0x0U << CMU_SCCR_SYSCLKSEL_Pos)
#define    FL_CMU_SYSTEM_CLK_SOURCE_XTHF                          (0x1U << CMU_SCCR_SYSCLKSEL_Pos)
#define    FL_CMU_SYSTEM_CLK_SOURCE_PLL                           (0x2U << CMU_SCCR_SYSCLKSEL_Pos)
#define    FL_CMU_SYSTEM_CLK_SOURCE_XTLF                          (0x5U << CMU_SCCR_SYSCLKSEL_Pos)
#define    FL_CMU_SYSTEM_CLK_SOURCE_RCLP                          (0x6U << CMU_SCCR_SYSCLKSEL_Pos)


#define    FL_CMU_RCHF_FREQUENCY_8MHZ                             (0x0U << CMU_RCHFCR_FSEL_Pos)
#define    FL_CMU_RCHF_FREQUENCY_16MHZ                            (0x1U << CMU_RCHFCR_FSEL_Pos)
#define    FL_CMU_RCHF_FREQUENCY_24MHZ                            (0x2U << CMU_RCHFCR_FSEL_Pos)
#define    FL_CMU_RCHF_FREQUENCY_32MHZ                            (0x3U << CMU_RCHFCR_FSEL_Pos)


#define    FL_CMU_PLL_PSC_DIV1                                    (0x0U << CMU_PLLCR_REFPRSC_Pos)
#define    FL_CMU_PLL_PSC_DIV2                                    (0x1U << CMU_PLLCR_REFPRSC_Pos)
#define    FL_CMU_PLL_PSC_DIV4                                    (0x2U << CMU_PLLCR_REFPRSC_Pos)
#define    FL_CMU_PLL_PSC_DIV8                                    (0x3U << CMU_PLLCR_REFPRSC_Pos)
#define    FL_CMU_PLL_PSC_DIV12                                   (0x4U << CMU_PLLCR_REFPRSC_Pos)
#define    FL_CMU_PLL_PSC_DIV16                                   (0x5U << CMU_PLLCR_REFPRSC_Pos)
#define    FL_CMU_PLL_PSC_DIV24                                   (0x6U << CMU_PLLCR_REFPRSC_Pos)
#define    FL_CMU_PLL_PSC_DIV32                                   (0x7U << CMU_PLLCR_REFPRSC_Pos)


#define    FL_CMU_PLL_OUTPUT_X1                                   (0x0U << CMU_PLLCR_OSEL_Pos)
#define    FL_CMU_PLL_OUTPUT_X2                                   (0x1U << CMU_PLLCR_OSEL_Pos)


#define    FL_CMU_PLL_CLK_SOURCE_RCHF                             (0x0U << CMU_PLLCR_INSEL_Pos)
#define    FL_CMU_PLL_CLK_SOURCE_XTHF                             (0x1U << CMU_PLLCR_INSEL_Pos)


#define    FL_CMU_XTLF_START_WAIT_CYCLE_128                       (0x0U << CMU_XTLFCR_WAIT_Pos)
#define    FL_CMU_XTLF_START_WAIT_CYCLE_256                       (0x1U << CMU_XTLFCR_WAIT_Pos)
#define    FL_CMU_XTLF_START_WAIT_CYCLE_512                       (0x2U << CMU_XTLFCR_WAIT_Pos)
#define    FL_CMU_XTLF_START_WAIT_CYCLE_1024                      (0x3U << CMU_XTLFCR_WAIT_Pos)
#define    FL_CMU_XTLF_START_WAIT_CYCLE_2048                      (0x4U << CMU_XTLFCR_WAIT_Pos)
#define    FL_CMU_XTLF_START_WAIT_CYCLE_4096                      (0x5U << CMU_XTLFCR_WAIT_Pos)
#define    FL_CMU_XTLF_START_WAIT_CYCLE_8192                      (0x6U << CMU_XTLFCR_WAIT_Pos)
#define    FL_CMU_XTLF_START_WAIT_CYCLE_16384                     (0x7U << CMU_XTLFCR_WAIT_Pos)


#define    FL_CMU_XTLF_FDET_ENABLE                                (0x5U << CMU_XTLFCR_EN_Pos)

#define    FL_CMU_XTLF_FDET_DISABLE                               (0xaU << CMU_XTLFCR_EN_Pos)

#define    FL_CMU_XTLF_WORK_CURRENT_850NA                         (0x0U << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_800NA                         (0x1U << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_750NA                         (0x2U << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_700NA                         (0x3U << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_650NA                         (0x4U << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_600NA                         (0x5U << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_550NA                         (0x6U << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_500NA                         (0x7U << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_450NA                         (0x8U << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_400NA                         (0x9U << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_350NA                         (0xaU << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_300NA                         (0xbU << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_250NA                         (0xcU << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_200NA                         (0xdU << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_150NA                         (0xeU << CMU_XTLFCR_IPW_Pos)
#define    FL_CMU_XTLF_WORK_CURRENT_100NA                         (0xfU << CMU_XTLFCR_IPW_Pos)


#define    FL_CMU_LSCLK_SOURCE_XTLF                               (0x0U << CMU_LSSR_LSSTA_Pos)
#define    FL_CMU_LSCLK_SOURCE_RCLP                               (0x1U << CMU_LSSR_LSSTA_Pos)

#define    FL_CMU_SET_LSCLK_SOURCE_RCLP                           (0x55U << CMU_LSSR_SEL_Pos)
#define    FL_CMU_SET_LSCLK_SOURCE_XTLF                           (0xAAU << CMU_LSSR_SEL_Pos)

#define    FL_CMU_XTHF_START_WAIT_CYCLE_128                       (0x0U << CMU_XTHFCR_WAIT_Pos)
#define    FL_CMU_XTHF_START_WAIT_CYCLE_256                       (0x1U << CMU_XTHFCR_WAIT_Pos)
#define    FL_CMU_XTHF_START_WAIT_CYCLE_512                       (0x2U << CMU_XTHFCR_WAIT_Pos)
#define    FL_CMU_XTHF_START_WAIT_CYCLE_1024                      (0x3U << CMU_XTHFCR_WAIT_Pos)
#define    FL_CMU_XTHF_START_WAIT_CYCLE_2048                      (0x4U << CMU_XTHFCR_WAIT_Pos)
#define    FL_CMU_XTHF_START_WAIT_CYCLE_4096                      (0x5U << CMU_XTHFCR_WAIT_Pos)
#define    FL_CMU_XTHF_START_WAIT_CYCLE_8192                      (0x6U << CMU_XTHFCR_WAIT_Pos)
#define    FL_CMU_XTHF_START_WAIT_CYCLE_16384                     (0x7U << CMU_XTHFCR_WAIT_Pos)


#define    FL_CMU_EXTI_CLK_SOURCE_LSCLK                           (0x1U << CMU_OPCCR1_EXTICKS_Pos)
#define    FL_CMU_EXTI_CLK_SOURCE_HCLK                            (0x0U << CMU_OPCCR1_EXTICKS_Pos)


#define    FL_CMU_LPUART1_CLK_SOURCE_LSCLK                        (0x0U << CMU_OPCCR1_LPUARTCKS_Pos)
#define    FL_CMU_LPUART1_CLK_SOURCE_RCHF                         (0x1U << CMU_OPCCR1_LPUARTCKS_Pos)


#define    FL_CMU_LPUART0_CLK_SOURCE_LSCLK                        (0x0U << CMU_OPCCR1_LPUARTCKS_Pos)
#define    FL_CMU_LPUART0_CLK_SOURCE_RCHF                         (0x1U << CMU_OPCCR1_LPUARTCKS_Pos)


#define    FL_CMU_I2CSMB1_CLK_SOURCE_APBCLK                       (0x0U << CMU_OPCCR1_I2CSMB1_Pos)
#define    FL_CMU_I2CSMB1_CLK_SOURCE_RCHF                         (0x1U << CMU_OPCCR1_I2CSMB1_Pos)
#define    FL_CMU_I2CSMB1_CLK_SOURCE_SYSCLK                       (0x2U << CMU_OPCCR1_I2CSMB1_Pos)


#define    FL_CMU_I2CSMB0_CLK_SOURCE_APBCLK                       (0x0U << CMU_OPCCR1_I2CSMB0_Pos)
#define    FL_CMU_I2CSMB0_CLK_SOURCE_RCHF                         (0x1U << CMU_OPCCR1_I2CSMB0_Pos)
#define    FL_CMU_I2CSMB0_CLK_SOURCE_SYSCLK                       (0x2U << CMU_OPCCR1_I2CSMB0_Pos)


#define    FL_CMU_BSTIM16_CLK_SOURCE_APBCLK                       (0x0U << CMU_OPCCR1_BT16CKS_Pos)
#define    FL_CMU_BSTIM16_CLK_SOURCE_LSCLK                        (0x1U << CMU_OPCCR1_BT16CKS_Pos)
#define    FL_CMU_BSTIM16_CLK_SOURCE_RCLP                         (0x2U << CMU_OPCCR1_BT16CKS_Pos)


#define    FL_CMU_LPTIM16_CLK_SOURCE_APBCLK                       (0x0U << CMU_OPCCR1_LPT16CKS_Pos)
#define    FL_CMU_LPTIM16_CLK_SOURCE_LSCLK                        (0x1U << CMU_OPCCR1_LPT16CKS_Pos)
#define    FL_CMU_LPTIM16_CLK_SOURCE_RCLP                         (0x2U << CMU_OPCCR1_LPT16CKS_Pos)


#define    FL_CMU_CAN1_CLK_SOURCE_RCHF                            (0x0U << CMU_OPCCR1_CAN1CKS_Pos)
#define    FL_CMU_CAN1_CLK_SOURCE_XTHF                            (0x1U << CMU_OPCCR1_CAN1CKS_Pos)
#define    FL_CMU_CAN1_CLK_SOURCE_PLL                             (0x2U << CMU_OPCCR1_CAN1CKS_Pos)
#define    FL_CMU_CAN1_CLK_SOURCE_APBCLK                          (0x3U << CMU_OPCCR1_CAN1CKS_Pos)


#define    FL_CMU_ATIM_CLK_SOURCE_APBCLK                          (0x0U << CMU_OPCCR1_ATCKS_Pos)
#define    FL_CMU_ATIM_CLK_SOURCE_PLL_X2                          (0x1U << CMU_OPCCR1_ATCKS_Pos)

#define    FL_CMU_CAN0_CLK_SOURCE_RCHF                            (0x0U << CMU_OPCCR1_CAN0CKS_Pos)
#define    FL_CMU_CAN0_CLK_SOURCE_XTHF                            (0x1U << CMU_OPCCR1_CAN0CKS_Pos)
#define    FL_CMU_CAN0_CLK_SOURCE_PLL                             (0x2U << CMU_OPCCR1_CAN0CKS_Pos)
#define    FL_CMU_CAN0_CLK_SOURCE_APBCLK                          (0x3U << CMU_OPCCR1_CAN0CKS_Pos)


#define    FL_CMU_UART1_CLK_SOURCE_APBCLK                         (0x0U << CMU_OPCCR1_UART1CKS_Pos)
#define    FL_CMU_UART1_CLK_SOURCE_RCHF                           (0x1U << CMU_OPCCR1_UART1CKS_Pos)
#define    FL_CMU_UART1_CLK_SOURCE_SYSCLK                         (0x2U << CMU_OPCCR1_UART1CKS_Pos)
#define    FL_CMU_UART1_CLK_SOURCE_XTHF                           (0x3U << CMU_OPCCR1_UART1CKS_Pos)


#define    FL_CMU_UART0_CLK_SOURCE_APBCLK                         (0x0U << CMU_OPCCR1_UART0CKS_Pos)
#define    FL_CMU_UART0_CLK_SOURCE_RCHF                           (0x1U << CMU_OPCCR1_UART0CKS_Pos)
#define    FL_CMU_UART0_CLK_SOURCE_SYSCLK                         (0x2U << CMU_OPCCR1_UART0CKS_Pos)
#define    FL_CMU_UART0_CLK_SOURCE_XTHF                           (0x3U << CMU_OPCCR1_UART0CKS_Pos)


#define    FL_CMU_RTC_CLK_SOURCE_LSCLK                            (0x0U << CMU_OPCCR2_RTCCKS_Pos)
#define    FL_CMU_RTC_CLK_SOURCE_XTHFPre                          (0x1U << CMU_OPCCR2_RTCCKS_Pos)


#define    FL_CMU_RNG_PSC_DIV1                                    (0x0U << CMU_OPCCR2_RNGPRSC_Pos)
#define    FL_CMU_RNG_PSC_DIV2                                    (0x1U << CMU_OPCCR2_RNGPRSC_Pos)
#define    FL_CMU_RNG_PSC_DIV4                                    (0x2U << CMU_OPCCR2_RNGPRSC_Pos)
#define    FL_CMU_RNG_PSC_DIV8                                    (0x3U << CMU_OPCCR2_RNGPRSC_Pos)
#define    FL_CMU_RNG_PSC_DIV16                                   (0x4U << CMU_OPCCR2_RNGPRSC_Pos)
#define    FL_CMU_RNG_PSC_DIV32                                   (0x5U << CMU_OPCCR2_RNGPRSC_Pos)


#define    FL_CMU_ADC_PSC_DIV1                                    (0x0U << CMU_OPCCR2_ADCPRSC_Pos)
#define    FL_CMU_ADC_PSC_DIV2                                    (0x1U << CMU_OPCCR2_ADCPRSC_Pos)
#define    FL_CMU_ADC_PSC_DIV4                                    (0x2U << CMU_OPCCR2_ADCPRSC_Pos)
#define    FL_CMU_ADC_PSC_DIV8                                    (0x3U << CMU_OPCCR2_ADCPRSC_Pos)
#define    FL_CMU_ADC_PSC_DIV16                                   (0x4U << CMU_OPCCR2_ADCPRSC_Pos)
#define    FL_CMU_ADC_PSC_DIV32                                   (0x5U << CMU_OPCCR2_ADCPRSC_Pos)


#define    FL_CMU_ADC_CLK_SOURCE_RCHF                             (0x1U << CMU_OPCCR2_ADCCKS_Pos)
#define    FL_CMU_ADC_CLK_SOURCE_XTHF                             (0x2U << CMU_OPCCR2_ADCCKS_Pos)
#define    FL_CMU_ADC_CLK_SOURCE_PLL                              (0x3U << CMU_OPCCR2_ADCCKS_Pos)


#define    FL_CMU_AHB_MASTER_PRIORITY_DMA                         (0x0U << CMU_AMCR_MPRIL_Pos)
#define    FL_CMU_AHB_MASTER_PRIORITY_CPU                         (0x1U << CMU_AMCR_MPRIL_Pos)


#define    FL_CMU_CCL_CALCLK_PSC_DIV1                             (0x0U << CMU_CCFR_CALPSC_Pos)
#define    FL_CMU_CCL_CALCLK_PSC_DIV2                             (0x1U << CMU_CCFR_CALPSC_Pos)
#define    FL_CMU_CCL_CALCLK_PSC_DIV4                             (0x2U << CMU_CCFR_CALPSC_Pos)
#define    FL_CMU_CCL_CALCLK_PSC_DIV8                             (0x3U << CMU_CCFR_CALPSC_Pos)


#define    FL_CMU_CCL_REFCLK_PSC_DIV8                             (0x0U << CMU_CCFR_REFPSC_Pos)
#define    FL_CMU_CCL_REFCLK_PSC_DIV16                            (0x1U << CMU_CCFR_REFPSC_Pos)
#define    FL_CMU_CCL_REFCLK_PSC_DIV32                            (0x2U << CMU_CCFR_REFPSC_Pos)
#define    FL_CMU_CCL_REFCLK_PSC_DIV64                            (0x3U << CMU_CCFR_REFPSC_Pos)


#define    FL_CMU_CCL_CALCLK_SOURCE_RCHF                          (0x1U << CMU_CCFR_CALSEL_Pos)
#define    FL_CMU_CCL_CALCLK_SOURCE_XTHF                          (0x3U << CMU_CCFR_CALSEL_Pos)


#define    FL_CMU_CCL_REFCLK_SOURCE_XTLF                          (0x0U << CMU_CCFR_REFSEL_Pos)
#define    FL_CMU_CCL_REFCLK_SOURCE_RCLP                          (0x1U << CMU_CCFR_REFSEL_Pos)


/**
  * @}
  */
/* Exported functions ---------------------------------------------------------------------------------*/
/** @defgroup CMU_FL_Exported_Functions CMU Exported Functions
  * @{
  */

/**
  * @brief    Enable LSCLK Auto Switch
  * @rmtoll   SYSCLKCR    LSCATS    FL_CMU_EnableLSCLKAutoSwitch
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_EnableLSCLKAutoSwitch(void)
{
    SET_BIT(CMU->SYSCLKCR, CMU_SCCR_LSCATS_Msk);
}

/**
  * @brief    Get LSCLK Auto Switch Enable Status
  * @rmtoll   SYSCLKCR    LSCATS    FL_CMU_IsEnabledLSCLKAutoSwitch
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsEnabledLSCLKAutoSwitch(void)
{
    return (uint32_t)(READ_BIT(CMU->SYSCLKCR, CMU_SCCR_LSCATS_Msk) == CMU_SCCR_LSCATS_Msk);
}

/**
  * @brief    Disable LSCLK Auto Switch
  * @rmtoll   SYSCLKCR    LSCATS    FL_CMU_DisableLSCLKAutoSwitch
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_DisableLSCLKAutoSwitch(void)
{
    CLEAR_BIT(CMU->SYSCLKCR, CMU_SCCR_LSCATS_Msk);
}

/**
  * @brief    Enable Sleep/DeepSleep Mode External Interrupt
  * @rmtoll   SYSCLKCR    SLP_ENEXTI    FL_CMU_EnableEXTIOnSleep
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_EnableEXTIOnSleep(void)
{
    SET_BIT(CMU->SYSCLKCR, CMU_SCCR_SLP_ENEXTI_Msk);
}

/**
  * @brief    Get Sleep/DeepSleep Mode External Interrupt Enable Status
  * @rmtoll   SYSCLKCR    SLP_ENEXTI    FL_CMU_IsEnabledEXTIOnSleep
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsEnabledEXTIOnSleep(void)
{
    return (uint32_t)(READ_BIT(CMU->SYSCLKCR, CMU_SCCR_SLP_ENEXTI_Msk) == CMU_SCCR_SLP_ENEXTI_Msk);
}

/**
  * @brief    Disable Sleep/DeepSleep Mode External Interrupt
  * @rmtoll   SYSCLKCR    SLP_ENEXTI    FL_CMU_DisableEXTIOnSleep
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_DisableEXTIOnSleep(void)
{
    CLEAR_BIT(CMU->SYSCLKCR, CMU_SCCR_SLP_ENEXTI_Msk);
}

/**
  * @brief    Set APB Prescaler
  * @rmtoll   SYSCLKCR    APBPRES    FL_CMU_SetAPBPrescaler
  * @param    prescaler This parameter can be one of the following values:
  *           @arg @ref FL_CMU_APBCLK_PSC_DIV1
  *           @arg @ref FL_CMU_APBCLK_PSC_DIV2
  *           @arg @ref FL_CMU_APBCLK_PSC_DIV4
  *           @arg @ref FL_CMU_APBCLK_PSC_DIV8
  *           @arg @ref FL_CMU_APBCLK_PSC_DIV16
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetAPBPrescaler(uint32_t prescaler)
{
    MODIFY_REG(CMU->SYSCLKCR, CMU_SCCR_APBPRES_Msk, prescaler);
}

/**
  * @brief    Get APB Prescaler
  * @rmtoll   SYSCLKCR    APBPRES    FL_CMU_GetAPBPrescaler
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_APBCLK_PSC_DIV1
  *           @arg @ref FL_CMU_APBCLK_PSC_DIV2
  *           @arg @ref FL_CMU_APBCLK_PSC_DIV4
  *           @arg @ref FL_CMU_APBCLK_PSC_DIV8
  *           @arg @ref FL_CMU_APBCLK_PSC_DIV16
  */
__STATIC_INLINE uint32_t FL_CMU_GetAPBPrescaler(void)
{
    return (uint32_t)(READ_BIT(CMU->SYSCLKCR, CMU_SCCR_APBPRES_Msk));
}

/**
  * @brief    Set AHB Prescaler
  * @rmtoll   SYSCLKCR    AHBPRES    FL_CMU_SetAHBPrescaler
  * @param    prescaler This parameter can be one of the following values:
  *           @arg @ref FL_CMU_AHBCLK_PSC_DIV1
  *           @arg @ref FL_CMU_AHBCLK_PSC_DIV2
  *           @arg @ref FL_CMU_AHBCLK_PSC_DIV4
  *           @arg @ref FL_CMU_AHBCLK_PSC_DIV8
  *           @arg @ref FL_CMU_AHBCLK_PSC_DIV16
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetAHBPrescaler(uint32_t prescaler)
{
    MODIFY_REG(CMU->SYSCLKCR, CMU_SCCR_AHBPRES_Msk, prescaler);
}

/**
  * @brief    Get AHB Prescaler
  * @rmtoll   SYSCLKCR    AHBPRES    FL_CMU_GetAHBPrescaler
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_AHBCLK_PSC_DIV1
  *           @arg @ref FL_CMU_AHBCLK_PSC_DIV2
  *           @arg @ref FL_CMU_AHBCLK_PSC_DIV4
  *           @arg @ref FL_CMU_AHBCLK_PSC_DIV8
  *           @arg @ref FL_CMU_AHBCLK_PSC_DIV16
  */
__STATIC_INLINE uint32_t FL_CMU_GetAHBPrescaler(void)
{
    return (uint32_t)(READ_BIT(CMU->SYSCLKCR, CMU_SCCR_AHBPRES_Msk));
}

/**
  * @brief    Set SysTick Clock Source
  * @rmtoll   SYSCLKCR    STCLKSEL    FL_CMU_SetSysTickClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_SYSTICK_CLK_SOURCE_SCLK
  *           @arg @ref FL_CMU_SYSTICK_CLK_SOURCE_LSCLK
  *           @arg @ref FL_CMU_SYSTICK_CLK_SOURCE_SYSCLK
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetSysTickClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->SYSCLKCR, CMU_SCCR_STCLKSEL_Msk, clock);
}

/**
  * @brief    Get SysTick Clock Source
  * @rmtoll   SYSCLKCR    STCLKSEL    FL_CMU_GetSysTickClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_SYSTICK_CLK_SOURCE_SCLK
  *           @arg @ref FL_CMU_SYSTICK_CLK_SOURCE_LSCLK
  *           @arg @ref FL_CMU_SYSTICK_CLK_SOURCE_SYSCLK
  */
__STATIC_INLINE uint32_t FL_CMU_GetSysTickClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->SYSCLKCR, CMU_SCCR_STCLKSEL_Msk));
}

/**
  * @brief    Set System Clock Source
  * @rmtoll   SYSCLKCR    SYSCLKSEL    FL_CMU_SetSystemClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_SYSTEM_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_SYSTEM_CLK_SOURCE_XTHF
  *           @arg @ref FL_CMU_SYSTEM_CLK_SOURCE_PLL
  *           @arg @ref FL_CMU_SYSTEM_CLK_SOURCE_XTLF
  *           @arg @ref FL_CMU_SYSTEM_CLK_SOURCE_RCLP
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetSystemClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->SYSCLKCR, CMU_SCCR_SYSCLKSEL_Msk, clock);
}

/**
  * @brief    Set System Clock Source Setting
  * @rmtoll   SYSCLKCR    SYSCLKSEL    FL_CMU_GetSystemClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_SYSTEM_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_SYSTEM_CLK_SOURCE_XTHF
  *           @arg @ref FL_CMU_SYSTEM_CLK_SOURCE_PLL
  *           @arg @ref FL_CMU_SYSTEM_CLK_SOURCE_XTLF
  *           @arg @ref FL_CMU_SYSTEM_CLK_SOURCE_RCLP
  */
__STATIC_INLINE uint32_t FL_CMU_GetSystemClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->SYSCLKCR, CMU_SCCR_SYSCLKSEL_Msk));
}

/**
  * @brief    Set RCHF Frequency
  * @rmtoll   RCHFCR    FSEL    FL_CMU_RCHF_SetFrequency
  * @param    frequency This parameter can be one of the following values:
  *           @arg @ref FL_CMU_RCHF_FREQUENCY_8MHZ
  *           @arg @ref FL_CMU_RCHF_FREQUENCY_16MHZ
  *           @arg @ref FL_CMU_RCHF_FREQUENCY_24MHZ
  *           @arg @ref FL_CMU_RCHF_FREQUENCY_32MHZ
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_RCHF_SetFrequency(uint32_t frequency)
{
    MODIFY_REG(CMU->RCHFCR, CMU_RCHFCR_FSEL_Msk, frequency);
}

/**
  * @brief    Get RCHF Frequency Setting
  * @rmtoll   RCHFCR    FSEL    FL_CMU_RCHF_GetFrequency
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_RCHF_FREQUENCY_8MHZ
  *           @arg @ref FL_CMU_RCHF_FREQUENCY_16MHZ
  *           @arg @ref FL_CMU_RCHF_FREQUENCY_24MHZ
  *           @arg @ref FL_CMU_RCHF_FREQUENCY_32MHZ
  */
__STATIC_INLINE uint32_t FL_CMU_RCHF_GetFrequency(void)
{
    return (uint32_t)(READ_BIT(CMU->RCHFCR, CMU_RCHFCR_FSEL_Msk));
}

/**
  * @brief    Enable RCHF
  * @rmtoll   RCHFCR    EN    FL_CMU_RCHF_Enable
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_RCHF_Enable(void)
{
    SET_BIT(CMU->RCHFCR, CMU_RCHFCR_EN_Msk);
}

/**
  * @brief    Get RCHF Enable Status
  * @rmtoll   RCHFCR    EN    FL_CMU_RCHF_IsEnabled
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_RCHF_IsEnabled(void)
{
    return (uint32_t)(READ_BIT(CMU->RCHFCR, CMU_RCHFCR_EN_Msk) == CMU_RCHFCR_EN_Msk);
}

/**
  * @brief    Disable RCHF
  * @rmtoll   RCHFCR    EN    FL_CMU_RCHF_Disable
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_RCHF_Disable(void)
{
    CLEAR_BIT(CMU->RCHFCR, CMU_RCHFCR_EN_Msk);
}

/**
  * @brief    Set RCHF Freqency Trim Value
  * @rmtoll   RCHFTR    TRIM    FL_CMU_RCHF_WriteTrimValue
  * @param    value TrimValue The value of RCHF trim
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_RCHF_WriteTrimValue(uint32_t value)
{
    MODIFY_REG(CMU->RCHFTR, (0xffU << 0U), (value << 0U));
}

/**
  * @brief    Get RCHF Freqency Trim Value
  * @rmtoll   RCHFTR    TRIM    FL_CMU_RCHF_ReadTrimValue
  * @retval   The value of RCHF trim
  */
__STATIC_INLINE uint32_t FL_CMU_RCHF_ReadTrimValue(void)
{
    return (uint32_t)(READ_BIT(CMU->RCHFTR, (0xffU << 0U)) >> 0U);
}

/**
  * @brief    Set PLL Multiplier
  * @rmtoll   PLLCR    DB    FL_CMU_PLL_WriteMultiplier
  * @param    multiplier
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_PLL_WriteMultiplier(uint32_t multiplier)
{
    MODIFY_REG(CMU->PLLCR, (0x3fU << 16U), (multiplier << 16U));
}

/**
  * @brief    Get PLL Multiplier Setting
  * @rmtoll   PLLCR    DB    FL_CMU_PLL_ReadMultiplier
  * @retval
  */
__STATIC_INLINE uint32_t FL_CMU_PLL_ReadMultiplier(void)
{
    return (uint32_t)(READ_BIT(CMU->PLLCR, (0x3fU << 16U)) >> 16U);
}

/**
  * @brief    Get PLL Ready Status
  * @rmtoll   PLLCR    LOCKED    FL_CMU_IsActiveFlag_PLLReady
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsActiveFlag_PLLReady(void)
{
    return (uint32_t)(READ_BIT(CMU->PLLCR, CMU_PLLCR_LOCKED_Msk) == (CMU_PLLCR_LOCKED_Msk));
}

/**
  * @brief    Set PLL Prescaler
  * @rmtoll   PLLCR    REFPRSC    FL_CMU_PLL_SetPrescaler
  * @param    prescaler This parameter can be one of the following values:
  *           @arg @ref FL_CMU_PLL_PSC_DIV1
  *           @arg @ref FL_CMU_PLL_PSC_DIV2
  *           @arg @ref FL_CMU_PLL_PSC_DIV4
  *           @arg @ref FL_CMU_PLL_PSC_DIV8
  *           @arg @ref FL_CMU_PLL_PSC_DIV12
  *           @arg @ref FL_CMU_PLL_PSC_DIV16
  *           @arg @ref FL_CMU_PLL_PSC_DIV24
  *           @arg @ref FL_CMU_PLL_PSC_DIV32
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_PLL_SetPrescaler(uint32_t prescaler)
{
    MODIFY_REG(CMU->PLLCR, CMU_PLLCR_REFPRSC_Msk, prescaler);
}

/**
  * @brief    Get PLL Prescaler Setting
  * @rmtoll   PLLCR    REFPRSC    FL_CMU_PLL_GetPrescaler
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_PLL_PSC_DIV1
  *           @arg @ref FL_CMU_PLL_PSC_DIV2
  *           @arg @ref FL_CMU_PLL_PSC_DIV4
  *           @arg @ref FL_CMU_PLL_PSC_DIV8
  *           @arg @ref FL_CMU_PLL_PSC_DIV12
  *           @arg @ref FL_CMU_PLL_PSC_DIV16
  *           @arg @ref FL_CMU_PLL_PSC_DIV24
  *           @arg @ref FL_CMU_PLL_PSC_DIV32
  */
__STATIC_INLINE uint32_t FL_CMU_PLL_GetPrescaler(void)
{
    return (uint32_t)(READ_BIT(CMU->PLLCR, CMU_PLLCR_REFPRSC_Msk));
}

/**
  * @brief    Set PLL Digital Domain Output
  * @rmtoll   PLLCR    OSEL    FL_CMU_PLL_SetOutputMultiplier
  * @param    multiplier This parameter can be one of the following values:
  *           @arg @ref FL_CMU_PLL_OUTPUT_X1
  *           @arg @ref FL_CMU_PLL_OUTPUT_X2
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_PLL_SetOutputMultiplier(uint32_t multiplier)
{
    MODIFY_REG(CMU->PLLCR, CMU_PLLCR_OSEL_Msk, multiplier);
}

/**
  * @brief    Get PLL Digital Domain Output Setting
  * @rmtoll   PLLCR    OSEL    FL_CMU_PLL_GetOutputMultiplier
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_PLL_OUTPUT_X1
  *           @arg @ref FL_CMU_PLL_OUTPUT_X2
  */
__STATIC_INLINE uint32_t FL_CMU_PLL_GetOutputMultiplier(void)
{
    return (uint32_t)(READ_BIT(CMU->PLLCR, CMU_PLLCR_OSEL_Msk));
}

/**
  * @brief    Set PLL Input Source
  * @rmtoll   PLLCR    INSEL    FL_CMU_PLL_SetClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_PLL_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_PLL_CLK_SOURCE_XTHF
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_PLL_SetClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->PLLCR, CMU_PLLCR_INSEL_Msk, clock);
}

/**
  * @brief    Get PLL Input Source Setting
  * @rmtoll   PLLCR    INSEL    FL_CMU_PLL_GetClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_PLL_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_PLL_CLK_SOURCE_XTHF
  */
__STATIC_INLINE uint32_t FL_CMU_PLL_GetClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->PLLCR, CMU_PLLCR_INSEL_Msk));
}

/**
  * @brief    Enable PLL
  * @rmtoll   PLLCR    EN    FL_CMU_PLL_Enable
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_PLL_Enable(void)
{
    SET_BIT(CMU->PLLCR, CMU_PLLCR_EN_Msk);
}

/**
  * @brief    Disable PLL
  * @rmtoll   PLLCR    EN    FL_CMU_PLL_Disable
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_PLL_Disable(void)
{
    CLEAR_BIT(CMU->PLLCR, CMU_PLLCR_EN_Msk);
}

/**
  * @brief    Get PLL Enable Status
  * @rmtoll   PLLCR    EN    FL_CMU_PLL_IsEnabled
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_PLL_IsEnabled(void)
{
    return (uint32_t)(READ_BIT(CMU->PLLCR, CMU_PLLCR_EN_Msk) == CMU_PLLCR_EN_Msk);
}

/**
  * @brief    Set RCLP Enable
  * @rmtoll   RCLPCR    ENB    FL_CMU_RCLP_Enable
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_RCLP_Enable(void)
{
    SET_BIT(CMU->RCLPCR, CMU_RCLPCR_ENB_Msk);
}

/**
  * @brief    Get RCLP Enable Flag
  * @rmtoll   RCLPCR    ENB    FL_CMU_RCLP_IsEnabled
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_RCLP_IsEnabled(void)
{
    return (uint32_t)(READ_BIT(CMU->RCLPCR, CMU_RCLPCR_ENB_Msk) == CMU_RCLPCR_ENB_Msk);
}

/**
  * @brief    Set RCLP Disable
  * @rmtoll   RCLPCR    ENB    FL_CMU_RCLP_Disable
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_RCLP_Disable(void)
{
    CLEAR_BIT(CMU->RCLPCR, CMU_RCLPCR_ENB_Msk);
}

/**
  * @brief    Set RCLP Frequency Trim Value
  * @rmtoll   RCLPTR    TRIM    FL_CMU_RCLP_WriteTrimValue
  * @param    value TrimValue The value of RCLP trim
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_RCLP_WriteTrimValue(uint32_t value)
{
    MODIFY_REG(CMU->RCLPTR, (0xffU << 0U), (value << 0U));
}

/**
  * @brief    Get RCLP Frequency Trim Value
  * @rmtoll   RCLPTR    TRIM    FL_CMU_RCLP_ReadTrimValue
  * @retval   The Value of RCLP trim
  */
__STATIC_INLINE uint32_t FL_CMU_RCLP_ReadTrimValue(void)
{
    return (uint32_t)(READ_BIT(CMU->RCLPTR, (0xffU << 0U)) >> 0U);
}

/**
  * @brief    Set XTLF Start Wait Time
  * @rmtoll   XTLFCR    WAIT    FL_CMU_XTLF_SetStartWaitTime
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_128
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_256
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_512
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_1024
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_2048
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_4096
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_8192
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_16384
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_XTLF_SetStartWaitTime(uint32_t clock)
{
    MODIFY_REG(CMU->XTLFCR, CMU_XTLFCR_WAIT_Msk, clock);
}

/**
  * @brief    Get XTLF Start Wait Time
  * @rmtoll   XTLFCR    WAIT    FL_CMU_XTLF_GetStartWaitTime
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_128
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_256
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_512
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_1024
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_2048
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_4096
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_8192
  *           @arg @ref FL_CMU_XTLF_START_WAIT_CYCLE_16384
  */
__STATIC_INLINE uint32_t FL_CMU_XTLF_GetStartWaitTime(void)
{
    return (uint32_t)(READ_BIT(CMU->XTLFCR, CMU_XTLFCR_WAIT_Msk));
}

/**
  * @brief    Enable XT32KI
  * @rmtoll   XTLFCR    EXCKSEN    FL_CMU_Enable_XT32KI_Input
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_Enable_XT32KI_Input(void)
{
    SET_BIT(CMU->XTLFCR, CMU_XTLFCR_EXCKSEN_Msk);
}

/**
  * @brief    Disable XT32KI
  * @rmtoll   XTLFCR    EXCKSEN    FL_CMU_Disable_XT32KI_Input
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_Disable_XT32KI_Input(void)
{
    CLEAR_BIT(CMU->XTLFCR, CMU_XTLFCR_EXCKSEN_Msk);
}

/**
  * @brief    Get XT32KI Enable Status
  * @rmtoll   XTLFCR    EXCKSEN    FL_CMU_IsEnabled_XT32KI_Input
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsEnabled_XT32KI_Input(void)
{
    return (uint32_t)(READ_BIT(CMU->XTLFCR, CMU_XTLFCR_EXCKSEN_Msk) == CMU_XTLFCR_EXCKSEN_Msk);
}

/**
  * @brief    Enable XTLF
  * @rmtoll   XTLFCR    EN    FL_CMU_XTLF_Enable
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_XTLF_Enable(void)
{
    MODIFY_REG(CMU->XTLFCR, CMU_XTLFCR_EN_Msk, FL_CMU_XTLF_FDET_ENABLE);
}

/**
  * @brief    Disable XTLF
  * @rmtoll   XTLFCR    EN    FL_CMU_XTLF_Disable
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_XTLF_Disable(void)
{
    MODIFY_REG(CMU->XTLFCR, CMU_XTLFCR_EN_Msk, FL_CMU_XTLF_FDET_DISABLE);
}

/**
  * @brief    Get XTLF Ready Status
  * @rmtoll   XTLFCR    RDY    FL_CMU_XTLF_IsReady
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_XTLF_IsReady(void)
{
    return (uint32_t)(READ_BIT(CMU->XTLFCR, CMU_XTLFCR_RDY_Msk) == CMU_XTLFCR_RDY_Msk);
}

/**
  * @brief    Set XTLF Current
  * @rmtoll   XTLFCR    IPW    FL_CMU_XTLF_SetWorkCurrent
  * @param    current This parameter can be one of the following values:
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_850NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_800NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_750NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_700NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_650NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_600NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_550NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_500NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_450NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_400NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_350NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_300NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_250NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_200NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_150NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_100NA
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_XTLF_SetWorkCurrent(uint32_t current)
{
    MODIFY_REG(CMU->XTLFCR, CMU_XTLFCR_IPW_Msk, current);
}

/**
  * @brief    Get XTLF Current Setting
  * @rmtoll   XTLFCR    IPW    FL_CMU_XTLF_GetWorkCurrent
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_850NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_800NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_750NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_700NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_650NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_600NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_550NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_500NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_450NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_400NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_350NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_300NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_250NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_200NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_150NA
  *           @arg @ref FL_CMU_XTLF_WORK_CURRENT_100NA
  */
__STATIC_INLINE uint32_t FL_CMU_XTLF_GetWorkCurrent(void)
{
    return (uint32_t)(READ_BIT(CMU->XTLFCR, CMU_XTLFCR_IPW_Msk));
}

/**
  * @brief    Get LSCLK Source
  * @rmtoll   LSCLKSEL    LSSTA    FL_CMU_GetLSCLKClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_LSCLK_SOURCE_XTLF
  *           @arg @ref FL_CMU_LSCLK_SOURCE_RCLP
  */
__STATIC_INLINE uint32_t FL_CMU_GetLSCLKClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->LSCLKSEL, CMU_LSSR_LSSTA_Msk));
}

/**
  * @brief    Set LSCLK Clock Source
  * @rmtoll   LSCLKSEL    SEL    FL_CMU_SetLSCLKClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_LSCLK_SOURCE_RCLP
  *           @arg @ref FL_CMU_LSCLK_SOURCE_XTLF
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetLSCLKClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->LSCLKSEL, CMU_LSSR_SEL_Msk, clock);
}

/**
  * @brief    Set XTHF Internal  CAP
  * @rmtoll   XTHFCR    CAPCFG    FL_CMU_XTHF_SetInternalCap
  * @param    strength
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_XTHF_SetInternalCap(uint32_t strength)
{
    MODIFY_REG(CMU->XTHFCR, (0x7U << 16U), (strength << 16U));
}

/**
  * @brief    Get XTHF Internal  CAP
  * @rmtoll   XTHFCR    CAPCFG    FL_CMU_XTHF_GetInternalCap
  * @retval
  */
__STATIC_INLINE uint32_t FL_CMU_XTHF_GetInternalCap(void)
{
    return (uint32_t)(READ_BIT(CMU->XTHFCR, (0x7U << 16U)) >> 16U);
}

/**
  * @brief    Set XTHF Start Wait Time
  * @rmtoll   XTHFCR    WAIT    FL_CMU_XTHF_SetStartWaitTime
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_128
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_256
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_512
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_1024
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_2048
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_4096
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_8192
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_16384
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_XTHF_SetStartWaitTime(uint32_t clock)
{
    MODIFY_REG(CMU->XTHFCR, CMU_XTHFCR_WAIT_Msk, clock);
}

/**
  * @brief    Get XTHF Start Wait Time
  * @rmtoll   XTHFCR    WAIT    FL_CMU_XTHF_GetStartWaitTime
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_128
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_256
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_512
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_1024
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_2048
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_4096
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_8192
  *           @arg @ref FL_CMU_XTHF_START_WAIT_CYCLE_16384
  */
__STATIC_INLINE uint32_t FL_CMU_XTHF_GetStartWaitTime(void)
{
    return (uint32_t)(READ_BIT(CMU->XTHFCR, CMU_XTHFCR_WAIT_Msk));
}

/**
  * @brief    Set XTHF Oscillation Strength
  * @rmtoll   XTHFCR    CFG    FL_CMU_XTHF_WriteDriverStrength
  * @param    strength
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_XTHF_WriteDriverStrength(uint32_t strength)
{
    MODIFY_REG(CMU->XTHFCR, (0x1fU << 8U), (strength << 8U));
}

/**
  * @brief    Get XTHF Oscillation Strength Setting
  * @rmtoll   XTHFCR    CFG    FL_CMU_XTHF_ReadDriverStrength
  * @retval
  */
__STATIC_INLINE uint32_t FL_CMU_XTHF_ReadDriverStrength(void)
{
    return (uint32_t)(READ_BIT(CMU->XTHFCR, (0x1fU << 8U)) >> 8U);
}

/**
  * @brief    Get XTHF Ready Status
  * @rmtoll   XTHFCR    RDY    FL_CMU_XTHF_IsReady
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_XTHF_IsReady(void)
{
    return (uint32_t)(READ_BIT(CMU->XTHFCR, CMU_XTHFCR_RDY_Msk) == CMU_XTHFCR_RDY_Msk);
}

/**
  * @brief    Enable XTHF
  * @rmtoll   XTHFCR    EN    FL_CMU_XTHF_Enable
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_XTHF_Enable(void)
{
    SET_BIT(CMU->XTHFCR, CMU_XTHFCR_EN_Msk);
}

/**
  * @brief    Get XTHF Enable Status
  * @rmtoll   XTHFCR    EN    FL_CMU_XTHF_IsEnabled
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_XTHF_IsEnabled(void)
{
    return (uint32_t)(READ_BIT(CMU->XTHFCR, CMU_XTHFCR_EN_Msk) == CMU_XTHFCR_EN_Msk);
}

/**
  * @brief    Disable XTHF
  * @rmtoll   XTHFCR    EN    FL_CMU_XTHF_Disable
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_XTHF_Disable(void)
{
    CLEAR_BIT(CMU->XTHFCR, CMU_XTHFCR_EN_Msk);
}

/**
  * @brief    Enable SYSCKE Wrong Interrupt
  * @rmtoll   IER    SYSCKE_IE    FL_CMU_EnableIT_SYSCKEWrong
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_EnableIT_SYSCKEWrong(void)
{
    SET_BIT(CMU->IER, CMU_IER_SYSCKE_IE_Msk);
}

/**
  * @brief    Get SYSCKE Wrong Interrupt Enable Status
  * @rmtoll   IER    SYSCKE_IE    FL_CMU_IsEnabledIT_SYSCKEWrong
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsEnabledIT_SYSCKEWrong(void)
{
    return (uint32_t)(READ_BIT(CMU->IER, CMU_IER_SYSCKE_IE_Msk) == CMU_IER_SYSCKE_IE_Msk);
}

/**
  * @brief    Disable SYSCKE Wrong Interrupt
  * @rmtoll   IER    SYSCKE_IE    FL_CMU_DisableIT_SYSCKEWrong
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_DisableIT_SYSCKEWrong(void)
{
    CLEAR_BIT(CMU->IER, CMU_IER_SYSCKE_IE_Msk);
}

/**
  * @brief    Enable XTHF Fail Interrupt
  * @rmtoll   IER    HFDET_IE    FL_CMU_EnableIT_XTHFFail
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_EnableIT_XTHFFail(void)
{
    SET_BIT(CMU->IER, CMU_IER_HFDET_IE_Msk);
}

/**
  * @brief    Get XTHF Fail Interrupt Enable Status
  * @rmtoll   IER    HFDET_IE    FL_CMU_IsEnabledIT_XTHFFail
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsEnabledIT_XTHFFail(void)
{
    return (uint32_t)(READ_BIT(CMU->IER, CMU_IER_HFDET_IE_Msk) == CMU_IER_HFDET_IE_Msk);
}

/**
  * @brief    Disable XTHF Fail Interrupt
  * @rmtoll   IER    HFDET_IE    FL_CMU_DisableIT_XTHFFail
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_DisableIT_XTHFFail(void)
{
    CLEAR_BIT(CMU->IER, CMU_IER_HFDET_IE_Msk);
}

/**
  * @brief    Enable XTLF Fail Interrupt
  * @rmtoll   IER    LFDET_IE    FL_CMU_EnableIT_XTLFFail
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_EnableIT_XTLFFail(void)
{
    SET_BIT(CMU->IER, CMU_IER_LFDET_IE_Msk);
}

/**
  * @brief    Get XTLF Fail Interrupt Enable Status
  * @rmtoll   IER    LFDET_IE    FL_CMU_IsEnabledIT_XTLFFail
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsEnabledIT_XTLFFail(void)
{
    return (uint32_t)(READ_BIT(CMU->IER, CMU_IER_LFDET_IE_Msk) == CMU_IER_LFDET_IE_Msk);
}

/**
  * @brief    Disable XTLF Fail Interrupt
  * @rmtoll   IER    LFDET_IE    FL_CMU_DisableIT_XTLFFail
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_DisableIT_XTLFFail(void)
{
    CLEAR_BIT(CMU->IER, CMU_IER_LFDET_IE_Msk);
}

/**
  * @brief    Get XTHF Vibrating Output
  * @rmtoll   ISR    HFDETO    FL_CMU_IsActiveFlag_XTHFFailOutput
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsActiveFlag_XTHFFailOutput(void)
{
    return (uint32_t)(READ_BIT(CMU->ISR, CMU_ISR_HFDETO_Msk) == (CMU_ISR_HFDETO_Msk));
}

/**
  * @brief    Get XTLF Vibrating Output
  * @rmtoll   ISR    LFDETO    FL_CMU_IsActiveFlag_XTLFFailOutput
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsActiveFlag_XTLFFailOutput(void)
{
    return (uint32_t)(READ_BIT(CMU->ISR, CMU_ISR_LFDETO_Msk) == (CMU_ISR_LFDETO_Msk));
}

/**
  * @brief    Get SYSCKE Wrong Flag
  * @rmtoll   ISR    SYSCSE_IF    FL_CMU_IsActiveFlag_SYSCSEWrong
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsActiveFlag_SYSCSEWrong(void)
{
    return (uint32_t)(READ_BIT(CMU->ISR, CMU_ISR_SYSCSE_IF_Msk) == (CMU_ISR_SYSCSE_IF_Msk));
}

/**
  * @brief    Clear SYSCKE Wrong Flag
  * @rmtoll   ISR    SYSCSE_IF    FL_CMU_ClearFlag_SYSCSEWrong
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_ClearFlag_SYSCSEWrong(void)
{
    WRITE_REG(CMU->ISR, CMU_ISR_SYSCSE_IF_Msk);
}

/**
  * @brief    Get XTHF Vibrating Flag
  * @rmtoll   ISR    HFDET_IF    FL_CMU_IsActiveFlag_XTHFFail
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsActiveFlag_XTHFFail(void)
{
    return (uint32_t)(READ_BIT(CMU->ISR, CMU_ISR_HFDET_IF_Msk) == (CMU_ISR_HFDET_IF_Msk));
}

/**
  * @brief    Clear XTHF Vibrating Flag
  * @rmtoll   ISR    HFDET_IF    FL_CMU_ClearFlag_XTHFFail
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_ClearFlag_XTHFFail(void)
{
    WRITE_REG(CMU->ISR, CMU_ISR_HFDET_IF_Msk);
}

/**
  * @brief    Get XTLF Vibrating Flag
  * @rmtoll   ISR    LFDET_IF    FL_CMU_IsActiveFlag_XTLFFail
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsActiveFlag_XTLFFail(void)
{
    return (uint32_t)(READ_BIT(CMU->ISR, CMU_ISR_LFDET_IF_Msk) == (CMU_ISR_LFDET_IF_Msk));
}

/**
  * @brief    Clear XTLF Vibrating Flag
  * @rmtoll   ISR    LFDET_IF    FL_CMU_ClearFlag_XTLFFail
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_ClearFlag_XTLFFail(void)
{
    WRITE_REG(CMU->ISR, CMU_ISR_LFDET_IF_Msk);
}

/**
  * @brief    Enable Group1 Periph Bus Clock
  * @rmtoll   PCLKCR1        FL_CMU_EnableGroup1BusClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_LPTIM16
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_RTC
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_PMU
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_SCU
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_IWDT
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_ANAC
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_PAD
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_SVD
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_COMP
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_EnableGroup1BusClock(uint32_t peripheral)
{
    SET_BIT(CMU->PCLKCR1, ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Enable Group2 Periph Bus Clock
  * @rmtoll   PCLKCR2        FL_CMU_EnableGroup2BusClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_CRC
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_RNG
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_AES
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_LCD
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_DMA
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_FLASH
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_RAMBIST
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_WWDT
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_ADC
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_DIVAS
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_TSC
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_PGL
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_INTMUX
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_EnableGroup2BusClock(uint32_t peripheral)
{
    SET_BIT(CMU->PCLKCR2, ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Enable Group3 Periph Bus Clock
  * @rmtoll   PCLKCR3        FL_CMU_EnableGroup3BusClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_SPI0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_SPI1
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_SPI2
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_SPI3
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART1
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART2
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART3
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART4
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART5
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UARTIR
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_LPUART
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_CAN0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_CAN1
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2C0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2C1
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2C2
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2C3
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2CSMB0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2CSMB1
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_EnableGroup3BusClock(uint32_t peripheral)
{
    SET_BIT(CMU->PCLKCR3, ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Enable Group4 Periph Bus Clock
  * @rmtoll   PCLKCR4        FL_CMU_EnableGroup4BusClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_GROUP4_BUSCLK_ATIM
  *           @arg @ref FL_CMU_GROUP4_BUSCLK_TAU0
  *           @arg @ref FL_CMU_GROUP4_BUSCLK_TAU1
  *           @arg @ref FL_CMU_GROUP4_BUSCLK_BSTIM16
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_EnableGroup4BusClock(uint32_t peripheral)
{
    SET_BIT(CMU->PCLKCR4, ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Disable Group1 Periph Bus Clock
  * @rmtoll   PCLKCR1        FL_CMU_DisableGroup1BusClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_LPTIM16
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_RTC
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_PMU
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_SCU
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_IWDT
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_ANAC
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_PAD
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_SVD
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_COMP
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_DisableGroup1BusClock(uint32_t peripheral)
{
    CLEAR_BIT(CMU->PCLKCR1, ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Disable Group2 Periph Bus Clock
  * @rmtoll   PCLKCR2        FL_CMU_DisableGroup2BusClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_CRC
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_RNG
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_AES
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_LCD
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_DMA
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_FLASH
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_RAMBIST
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_WWDT
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_ADC
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_DIVAS
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_TSC
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_PGL
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_INTMUX
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_DisableGroup2BusClock(uint32_t peripheral)
{
    CLEAR_BIT(CMU->PCLKCR2, ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Disable Group3 Periph Bus Clock
  * @rmtoll   PCLKCR3        FL_CMU_DisableGroup3BusClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_SPI0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_SPI1
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_SPI2
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_SPI3
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART1
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART2
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART3
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART4
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART5
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UARTIR
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_LPUART
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_CAN0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_CAN1
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2C0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2C1
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2C2
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2C3
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2CSMB0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2CSMB1
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_DisableGroup3BusClock(uint32_t peripheral)
{
    CLEAR_BIT(CMU->PCLKCR3, ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Disable Group4 Periph Bus Clock
  * @rmtoll   PCLKCR4        FL_CMU_DisableGroup4BusClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_GROUP4_BUSCLK_ATIM
  *           @arg @ref FL_CMU_GROUP4_BUSCLK_TAU0
  *           @arg @ref FL_CMU_GROUP4_BUSCLK_TAU1
  *           @arg @ref FL_CMU_GROUP4_BUSCLK_BSTIM16
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_DisableGroup4BusClock(uint32_t peripheral)
{
    CLEAR_BIT(CMU->PCLKCR4, ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Get Group1 Periph Bus Clock Enable Status
  * @rmtoll   PCLKCR1        FL_CMU_IsEnabledGroup1BusClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_LPTIM16
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_RTC
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_PMU
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_SCU
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_IWDT
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_ANAC
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_PAD
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_SVD
  *           @arg @ref FL_CMU_GROUP1_BUSCLK_COMP
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsEnabledGroup1BusClock(uint32_t peripheral)
{
    return (uint32_t)(READ_BIT(CMU->PCLKCR1, ((peripheral & 0xffffffff) << 0x0U)) == ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Get Group2 Periph Bus Clock Enable Status
  * @rmtoll   PCLKCR2        FL_CMU_IsEnabledGroup2BusClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_CRC
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_RNG
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_AES
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_LCD
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_DMA
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_FLASH
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_RAMBIST
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_WWDT
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_ADC
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_DIVAS
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_TSC
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_PGL
  *           @arg @ref FL_CMU_GROUP2_BUSCLK_INTMUX
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsEnabledGroup2BusClock(uint32_t peripheral)
{
    return (uint32_t)(READ_BIT(CMU->PCLKCR2, ((peripheral & 0xffffffff) << 0x0U)) == ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Get Group3 Periph Bus Clock Enable Status
  * @rmtoll   PCLKCR3        FL_CMU_IsEnabledGroup3BusClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_SPI0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_SPI1
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_SPI2
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_SPI3
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART1
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART2
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART3
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART4
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UART5
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_UARTIR
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_LPUART
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_CAN0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_CAN1
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2C0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2C1
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2C2
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2C3
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2CSMB0
  *           @arg @ref FL_CMU_GROUP3_BUSCLK_I2CSMB1
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsEnabledGroup3BusClock(uint32_t peripheral)
{
    return (uint32_t)(READ_BIT(CMU->PCLKCR3, ((peripheral & 0xffffffff) << 0x0U)) == ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Get Group4 Periph Bus Clock Enable Status
  * @rmtoll   PCLKCR4        FL_CMU_IsEnabledGroup4BusClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_GROUP4_BUSCLK_ATIM
  *           @arg @ref FL_CMU_GROUP4_BUSCLK_TAU0
  *           @arg @ref FL_CMU_GROUP4_BUSCLK_TAU1
  *           @arg @ref FL_CMU_GROUP4_BUSCLK_BSTIM16
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsEnabledGroup4BusClock(uint32_t peripheral)
{
    return (uint32_t)(READ_BIT(CMU->PCLKCR4, ((peripheral & 0xffffffff) << 0x0U)) == ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Enable Periph Operation Clock
  * @rmtoll   OPCCR3        FL_CMU_EnableOperationClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_OPCLK_EXTI
  *           @arg @ref FL_CMU_OPCLK_FLASH
  *           @arg @ref FL_CMU_OPCLK_LPUART
  *           @arg @ref FL_CMU_OPCLK_RNG
  *           @arg @ref FL_CMU_OPCLK_I2CSMB1
  *           @arg @ref FL_CMU_OPCLK_I2CSMB0
  *           @arg @ref FL_CMU_OPCLK_ADC
  *           @arg @ref FL_CMU_OPCLK_ATIM
  *           @arg @ref FL_CMU_OPCLK_CAN1
  *           @arg @ref FL_CMU_OPCLK_CAN0
  *           @arg @ref FL_CMU_OPCLK_LIN
  *           @arg @ref FL_CMU_OPCLK_UART1
  *           @arg @ref FL_CMU_OPCLK_UART0
  *           @arg @ref FL_CMU_OPCLK_BSTIM16
  *           @arg @ref FL_CMU_OPCLK_LPTIM16
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_EnableOperationClock(uint32_t peripheral)
{
    SET_BIT(CMU->OPCCR3, ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Disable Periph Operation Clock
  * @rmtoll   OPCCR3        FL_CMU_DisableOperationClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_OPCLK_EXTI
  *           @arg @ref FL_CMU_OPCLK_FLASH
  *           @arg @ref FL_CMU_OPCLK_LPUART
  *           @arg @ref FL_CMU_OPCLK_RNG
  *           @arg @ref FL_CMU_OPCLK_I2CSMB1
  *           @arg @ref FL_CMU_OPCLK_I2CSMB0
  *           @arg @ref FL_CMU_OPCLK_ADC
  *           @arg @ref FL_CMU_OPCLK_ATIM
  *           @arg @ref FL_CMU_OPCLK_CAN1
  *           @arg @ref FL_CMU_OPCLK_CAN0
  *           @arg @ref FL_CMU_OPCLK_LIN
  *           @arg @ref FL_CMU_OPCLK_UART1
  *           @arg @ref FL_CMU_OPCLK_UART0
  *           @arg @ref FL_CMU_OPCLK_BSTIM16
  *           @arg @ref FL_CMU_OPCLK_LPTIM16
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_DisableOperationClock(uint32_t peripheral)
{
    CLEAR_BIT(CMU->OPCCR3, ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Get Periph Operation Clock Enable Status
  * @rmtoll   OPCCR3        FL_CMU_IsEnabledOperationClock
  * @param    peripheral This parameter can be one of the following values:
  *           @arg @ref FL_CMU_OPCLK_EXTI
  *           @arg @ref FL_CMU_OPCLK_FLASH
  *           @arg @ref FL_CMU_OPCLK_LPUART
  *           @arg @ref FL_CMU_OPCLK_RNG
  *           @arg @ref FL_CMU_OPCLK_I2CSMB1
  *           @arg @ref FL_CMU_OPCLK_I2CSMB0
  *           @arg @ref FL_CMU_OPCLK_ADC
  *           @arg @ref FL_CMU_OPCLK_ATIM
  *           @arg @ref FL_CMU_OPCLK_CAN1
  *           @arg @ref FL_CMU_OPCLK_CAN0
  *           @arg @ref FL_CMU_OPCLK_LIN
  *           @arg @ref FL_CMU_OPCLK_UART1
  *           @arg @ref FL_CMU_OPCLK_UART0
  *           @arg @ref FL_CMU_OPCLK_BSTIM16
  *           @arg @ref FL_CMU_OPCLK_LPTIM16
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsEnabledOperationClock(uint32_t peripheral)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR3, ((peripheral & 0xffffffff) << 0x0U)) == ((peripheral & 0xffffffff) << 0x0U));
}

/**
  * @brief    Set EXTI Clock Source
  * @rmtoll   OPCCR1    EXTICKS    FL_CMU_SetEXTIClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_EXTI_CLK_SOURCE_LSCLK
  *           @arg @ref FL_CMU_EXTI_CLK_SOURCE_HCLK
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetEXTIClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR1, CMU_OPCCR1_EXTICKS_Msk, clock);
}

/**
  * @brief    Get EXTI Clock Source Setting
  * @rmtoll   OPCCR1    EXTICKS    FL_CMU_GetEXTIClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_EXTI_CLK_SOURCE_LSCLK
  *           @arg @ref FL_CMU_EXTI_CLK_SOURCE_HCLK
  */
__STATIC_INLINE uint32_t FL_CMU_GetEXTIClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR1, CMU_OPCCR1_EXTICKS_Msk));
}

/**
  * @brief    Set LPUART Clock Source
  * @rmtoll   OPCCR1    LPUARTCKS    FL_CMU_SetLPUARTClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_LPUART1_CLK_SOURCE_LSCLK
  *           @arg @ref FL_CMU_LPUART1_CLK_SOURCE_RCHF
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetLPUARTClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR1, CMU_OPCCR1_LPUARTCKS_Msk, clock);
}

/**
  * @brief    Get LPUART Clock Source Setting
  * @rmtoll   OPCCR1    LPUARTCKS    FL_CMU_GetLPUARTClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_LPUART1_CLK_SOURCE_LSCLK
  *           @arg @ref FL_CMU_LPUART1_CLK_SOURCE_RCHF
  */
__STATIC_INLINE uint32_t FL_CMU_GetLPUARTClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR1, CMU_OPCCR1_LPUARTCKS_Msk));
}

/**
  * @brief    Set I2C_SMB1 Clock Source
  * @rmtoll   OPCCR1    I2CSMB1    FL_CMU_SetI2CSMB1ClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_I2CSMB1_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_I2CSMB1_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_I2CSMB1_CLK_SOURCE_SYSCLK
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetI2CSMB1ClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR1, CMU_OPCCR1_I2CSMB1_Msk, clock);
}

/**
  * @brief    Get I2C_SMB1 Source Setting
  * @rmtoll   OPCCR1    I2CSMB1    FL_CMU_GetI2CSMB1ClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_I2CSMB1_CLK_SOURCE_APBCLK
  */
__STATIC_INLINE uint32_t FL_CMU_GetI2CSMB1ClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR1, CMU_OPCCR1_I2CSMB1_Msk));
}

/**
  * @brief    Set I2C_SMB0 Clock Source
  * @rmtoll   OPCCR1    I2CSMB0    FL_CMU_SetI2CSMB0ClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_I2CSMB0_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_I2CSMB0_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_I2CSMB0_CLK_SOURCE_SYSCLK
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetI2CSMB0ClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR1, CMU_OPCCR1_I2CSMB0_Msk, clock);
}

/**
  * @brief    Get I2C_SMB0 Source Setting
  * @rmtoll   OPCCR1    I2CSMB0    FL_CMU_GetI2CSMB0ClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_I2CSMB0_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_I2CSMB0_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_I2CSMB0_CLK_SOURCE_SYSCLK
  */
__STATIC_INLINE uint32_t FL_CMU_GetI2CSMB0ClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR1, CMU_OPCCR1_I2CSMB0_Msk));
}

/**
  * @brief    Set BSTIM16 Clock Source
  * @rmtoll   OPCCR1    BT16CKS    FL_CMU_SetBSTIM16ClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_BSTIM16_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_BSTIM16_CLK_SOURCE_LSCLK
  *           @arg @ref FL_CMU_BSTIM16_CLK_SOURCE_RCLP
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetBSTIM16ClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR1, CMU_OPCCR1_BT16CKS_Msk, clock);
}

/**
  * @brief    Get BSTIM16 Clock Source Setting
  * @rmtoll   OPCCR1    BT16CKS    FL_CMU_GetBSTIM16ClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_BSTIM16_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_BSTIM16_CLK_SOURCE_LSCLK
  *           @arg @ref FL_CMU_BSTIM16_CLK_SOURCE_RCLP
  */
__STATIC_INLINE uint32_t FL_CMU_GetBSTIM16ClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR1, CMU_OPCCR1_BT16CKS_Msk));
}

/**
  * @brief    Set LPTIM16 Clock Source
  * @rmtoll   OPCCR1    LPT16CKS    FL_CMU_SetLPTIM16ClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_LPTIM16_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_LPTIM16_CLK_SOURCE_LSCLK
  *           @arg @ref FL_CMU_LPTIM16_CLK_SOURCE_RCLP
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetLPTIM16ClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR1, CMU_OPCCR1_LPT16CKS_Msk, clock);
}

/**
  * @brief    Get LPTIM16 Clock Source Setting
  * @rmtoll   OPCCR1    LPT16CKS    FL_CMU_GetLPTIM16ClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_LPTIM16_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_LPTIM16_CLK_SOURCE_LSCLK
  *           @arg @ref FL_CMU_LPTIM16_CLK_SOURCE_RCLP
  */
__STATIC_INLINE uint32_t FL_CMU_GetLPTIM16ClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR1, CMU_OPCCR1_LPT16CKS_Msk));
}

/**
  * @brief    Set CAN1 Clock Source
  * @rmtoll   OPCCR1    CAN1CKS    FL_CMU_SetCAN1ClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_CAN1_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_CAN1_CLK_SOURCE_XTHF
  *           @arg @ref FL_CMU_CAN1_CLK_SOURCE_PLL
  *           @arg @ref FL_CMU_CAN1_CLK_SOURCE_APBCLK
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetCAN1ClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR1, CMU_OPCCR1_CAN1CKS_Msk, clock);
}

/**
  * @brief    Get CAN1 Clock Source Setting
  * @rmtoll   OPCCR1    CAN1CKS    FL_CMU_GetCAN1ClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_CAN1_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_CAN1_CLK_SOURCE_XTHF
  *           @arg @ref FL_CMU_CAN1_CLK_SOURCE_PLL
  *           @arg @ref FL_CMU_CAN1_CLK_SOURCE_APBCLK
  */
__STATIC_INLINE uint32_t FL_CMU_GetCAN1ClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR1, CMU_OPCCR1_CAN1CKS_Msk));
}

/**
  * @brief    Set ATIM Clock Source
  * @rmtoll   OPCCR1    ATCKS    FL_CMU_SetATIMClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_ATIM_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_ATIM_CLK_SOURCE_PLL_X2
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetATIMClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR1, CMU_OPCCR1_ATCKS_Msk, clock);
}

/**
  * @brief    Get ATIM Clock Source Setting
  * @rmtoll   OPCCR1    ATCKS    FL_CMU_GetATIMClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_ATIM_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_ATIM_CLK_SOURCE_PLL_X2
  */
__STATIC_INLINE uint32_t FL_CMU_GetATIMClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR1, CMU_OPCCR1_ATCKS_Msk));
}

/**
  * @brief    Set CAN0 Clock Source
  * @rmtoll   OPCCR1    CAN0CKS    FL_CMU_SetCAN0ClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_CAN0_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_CAN0_CLK_SOURCE_XTHF
  *           @arg @ref FL_CMU_CAN0_CLK_SOURCE_PLL
  *           @arg @ref FL_CMU_CAN0_CLK_SOURCE_APBCLK
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetCAN0ClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR1, CMU_OPCCR1_CAN0CKS_Msk, clock);
}

/**
  * @brief    Get CAN0 Clock Source Setting
  * @rmtoll   OPCCR1    CAN0CKS    FL_CMU_GetCAN0ClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_CAN0_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_CAN0_CLK_SOURCE_XTHF
  *           @arg @ref FL_CMU_CAN0_CLK_SOURCE_PLL
  *           @arg @ref FL_CMU_CAN0_CLK_SOURCE_APBCLK
  */
__STATIC_INLINE uint32_t FL_CMU_GetCAN0ClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR1, CMU_OPCCR1_CAN0CKS_Msk));
}

/**
  * @brief    Set UART1 Clock Source
  * @rmtoll   OPCCR1    UART1CKS    FL_CMU_SetUART1ClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_UART1_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_UART1_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_UART1_CLK_SOURCE_SYSCLK
  *           @arg @ref FL_CMU_UART1_CLK_SOURCE_XTHF
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetUART1ClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR1, CMU_OPCCR1_UART1CKS_Msk, clock);
}

/**
  * @brief    Get UART1 Clock Source Setting
  * @rmtoll   OPCCR1    UART1CKS    FL_CMU_GetUART1ClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_UART1_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_UART1_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_UART1_CLK_SOURCE_SYSCLK
  *           @arg @ref FL_CMU_UART1_CLK_SOURCE_XTHF
  */
__STATIC_INLINE uint32_t FL_CMU_GetUART1ClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR1, CMU_OPCCR1_UART1CKS_Msk));
}

/**
  * @brief    Set UART0 Clock Source
  * @rmtoll   OPCCR1    UART0CKS    FL_CMU_SetUART0ClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_UART0_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_UART0_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_UART0_CLK_SOURCE_SYSCLK
  *           @arg @ref FL_CMU_UART0_CLK_SOURCE_XTHF
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetUART0ClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR1, CMU_OPCCR1_UART0CKS_Msk, clock);
}

/**
  * @brief    Get UART0 Clock Source Setting
  * @rmtoll   OPCCR1    UART0CKS    FL_CMU_GetUART0ClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_UART0_CLK_SOURCE_APBCLK
  *           @arg @ref FL_CMU_UART0_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_UART0_CLK_SOURCE_SYSCLK
  *           @arg @ref FL_CMU_UART0_CLK_SOURCE_XTHF
  */
__STATIC_INLINE uint32_t FL_CMU_GetUART0ClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR1, CMU_OPCCR1_UART0CKS_Msk));
}

/**
  * @brief    Set XTHF Prescaler for RTC
  * @rmtoll   OPCCR2    RTCPRSC    FL_CMU_SetRTC_Prescaler
  * @param    prescaler
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetRTC_Prescaler(uint32_t prescaler)
{
    MODIFY_REG(CMU->OPCCR2, (0x3ffU << 22U), (prescaler << 22U));
}

/**
  * @brief    Get XTHF Prescaler for RTC
  * @rmtoll   OPCCR2    RTCPRSC    FL_CMU_GetRTC_Prescaler
  * @retval
  */
__STATIC_INLINE uint32_t FL_CMU_GetRTC_Prescaler(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR2, (0x3ffU << 22U)) >> 22U);
}

/**
  * @brief    Set RTC Clock Source
  * @rmtoll   OPCCR2    RTCCKS    FL_CMU_SetRTCClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_RTC_CLK_SOURCE_LSCLK
  *           @arg @ref FL_CMU_RTC_CLK_SOURCE_XTHFPre
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetRTCClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR2, CMU_OPCCR2_RTCCKS_Msk, clock);
}

/**
  * @brief    Get RTC Clock Source
  * @rmtoll   OPCCR2    RTCCKS    FL_CMU_GetRTCClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_RTC_CLK_SOURCE_LSCLK
  *           @arg @ref FL_CMU_RTC_CLK_SOURCE_XTHFPre
  */
__STATIC_INLINE uint32_t FL_CMU_GetRTCClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR2, CMU_OPCCR2_RTCCKS_Msk));
}

/**
  * @brief    Set RNG Prescaler
  * @rmtoll   OPCCR2    RNGPRSC    FL_CMU_SetRNGPrescaler
  * @param    prescaler This parameter can be one of the following values:
  *           @arg @ref FL_CMU_RNG_PSC_DIV1
  *           @arg @ref FL_CMU_RNG_PSC_DIV2
  *           @arg @ref FL_CMU_RNG_PSC_DIV4
  *           @arg @ref FL_CMU_RNG_PSC_DIV8
  *           @arg @ref FL_CMU_RNG_PSC_DIV16
  *           @arg @ref FL_CMU_RNG_PSC_DIV32
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetRNGPrescaler(uint32_t prescaler)
{
    MODIFY_REG(CMU->OPCCR2, CMU_OPCCR2_RNGPRSC_Msk, prescaler);
}

/**
  * @brief    Get RNG Prescaler Setting
  * @rmtoll   OPCCR2    RNGPRSC    FL_CMU_GetRNGPrescaler
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_RNG_PSC_DIV1
  *           @arg @ref FL_CMU_RNG_PSC_DIV2
  *           @arg @ref FL_CMU_RNG_PSC_DIV4
  *           @arg @ref FL_CMU_RNG_PSC_DIV8
  *           @arg @ref FL_CMU_RNG_PSC_DIV16
  *           @arg @ref FL_CMU_RNG_PSC_DIV32
  */
__STATIC_INLINE uint32_t FL_CMU_GetRNGPrescaler(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR2, CMU_OPCCR2_RNGPRSC_Msk));
}

/**
  * @brief    Set ADC Prescaler
  * @rmtoll   OPCCR2    ADCPRSC    FL_CMU_SetADCPrescaler
  * @param    prescaler This parameter can be one of the following values:
  *           @arg @ref FL_CMU_ADC_PSC_DIV1
  *           @arg @ref FL_CMU_ADC_PSC_DIV2
  *           @arg @ref FL_CMU_ADC_PSC_DIV4
  *           @arg @ref FL_CMU_ADC_PSC_DIV8
  *           @arg @ref FL_CMU_ADC_PSC_DIV16
  *           @arg @ref FL_CMU_ADC_PSC_DIV32
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetADCPrescaler(uint32_t prescaler)
{
    MODIFY_REG(CMU->OPCCR2, CMU_OPCCR2_ADCPRSC_Msk, prescaler);
}

/**
  * @brief    Get ADC Prescaler Setting
  * @rmtoll   OPCCR2    ADCPRSC    FL_CMU_GetADCPrescaler
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_ADC_PSC_DIV1
  *           @arg @ref FL_CMU_ADC_PSC_DIV2
  *           @arg @ref FL_CMU_ADC_PSC_DIV4
  *           @arg @ref FL_CMU_ADC_PSC_DIV8
  *           @arg @ref FL_CMU_ADC_PSC_DIV16
  *           @arg @ref FL_CMU_ADC_PSC_DIV32
  */
__STATIC_INLINE uint32_t FL_CMU_GetADCPrescaler(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR2, CMU_OPCCR2_ADCPRSC_Msk));
}

/**
  * @brief    Set ADC Clock Source
  * @rmtoll   OPCCR2    ADCCKS    FL_CMU_SetADCClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_ADC_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_ADC_CLK_SOURCE_XTHF
  *           @arg @ref FL_CMU_ADC_CLK_SOURCE_PLL
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetADCClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->OPCCR2, CMU_OPCCR2_ADCCKS_Msk, clock);
}

/**
  * @brief    Get ADC Clock Source Setting
  * @rmtoll   OPCCR2    ADCCKS    FL_CMU_GetADCClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_ADC_CLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_ADC_CLK_SOURCE_XTHF
  *           @arg @ref FL_CMU_ADC_CLK_SOURCE_PLL
  */
__STATIC_INLINE uint32_t FL_CMU_GetADCClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->OPCCR2, CMU_OPCCR2_ADCCKS_Msk));
}

/**
  * @brief    Set AHB Master Priority
  * @rmtoll   AHBMCR    MPRIL    FL_CMU_SetAHBMasterPriority
  * @param    priority This parameter can be one of the following values:
  *           @arg @ref FL_CMU_AHB_MASTER_PRIORITY_DMA
  *           @arg @ref FL_CMU_AHB_MASTER_PRIORITY_CPU
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_SetAHBMasterPriority(uint32_t priority)
{
    MODIFY_REG(CMU->AHBMCR, CMU_AMCR_MPRIL_Msk, priority);
}

/**
  * @brief    Get AHB Master Priority
  * @rmtoll   AHBMCR    MPRIL    FL_CMU_GetAHBMasterPriority
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_AHB_MASTER_PRIORITY_DMA
  *           @arg @ref FL_CMU_AHB_MASTER_PRIORITY_CPU
  */
__STATIC_INLINE uint32_t FL_CMU_GetAHBMasterPriority(void)
{
    return (uint32_t)(READ_BIT(CMU->AHBMCR, CMU_AMCR_MPRIL_Msk));
}

/**
  * @brief    Disable CCL Interrupt
  * @rmtoll   CCCR    CCLIE    FL_CMU_CCL_DisableIT
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_CCL_DisableIT(void)
{
    CLEAR_BIT(CMU->CCCR, CMU_CCCR_CCLIE_Msk);
}

/**
  * @brief    Enable CCL Interrupt
  * @rmtoll   CCCR    CCLIE    FL_CMU_CCL_EnableIT
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_CCL_EnableIT(void)
{
    SET_BIT(CMU->CCCR, CMU_CCCR_CCLIE_Msk);
}

/**
  * @brief    Get CCL Interrupt Enable Status
  * @rmtoll   CCCR    CCLIE    FL_CMU_CCL_IsEnabledIT
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_CCL_IsEnabledIT(void)
{
    return (uint32_t)(READ_BIT(CMU->CCCR, CMU_CCCR_CCLIE_Msk) == CMU_CCCR_CCLIE_Msk);
}

/**
  * @brief    Enable CCL
  * @rmtoll   CCCR    EN    FL_CMU_CCL_Enable
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_CCL_Enable(void)
{
    SET_BIT(CMU->CCCR, CMU_CCCR_EN_Msk);
}

/**
  * @brief    Get CCL Enable Status
  * @rmtoll   CCCR    EN    FL_CMU_CCL_IsEnabled
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_CCL_IsEnabled(void)
{
    return (uint32_t)(READ_BIT(CMU->CCCR, CMU_CCCR_EN_Msk) == CMU_CCCR_EN_Msk);
}

/**
  * @brief    Disable CCL
  * @rmtoll   CCCR    EN    FL_CMU_CCL_Disable
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_CCL_Disable(void)
{
    CLEAR_BIT(CMU->CCCR, CMU_CCCR_EN_Msk);
}

/**
  * @brief    Set CCL Calibration Clock Prescaler
  * @rmtoll   CCFR    CALPSC    FL_CMU_CCL_SetCalibrationClockPrescaler
  * @param    prescaler This parameter can be one of the following values:
  *           @arg @ref FL_CMU_CCL_CALCLK_PSC_DIV1
  *           @arg @ref FL_CMU_CCL_CALCLK_PSC_DIV2
  *           @arg @ref FL_CMU_CCL_CALCLK_PSC_DIV4
  *           @arg @ref FL_CMU_CCL_CALCLK_PSC_DIV8
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_CCL_SetCalibrationClockPrescaler(uint32_t prescaler)
{
    MODIFY_REG(CMU->CCFR, CMU_CCFR_CALPSC_Msk, prescaler);
}

/**
  * @brief    Get CCL Calibration Clock Prescaler
  * @rmtoll   CCFR    CALPSC    FL_CMU_CCL_GetCalibrationClockPrescaler
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_CCL_CALCLK_PSC_DIV1
  *           @arg @ref FL_CMU_CCL_CALCLK_PSC_DIV2
  *           @arg @ref FL_CMU_CCL_CALCLK_PSC_DIV4
  *           @arg @ref FL_CMU_CCL_CALCLK_PSC_DIV8
  */
__STATIC_INLINE uint32_t FL_CMU_CCL_GetCalibrationClockPrescaler(void)
{
    return (uint32_t)(READ_BIT(CMU->CCFR, CMU_CCFR_CALPSC_Msk));
}

/**
  * @brief    Set CCL Reference Clock Prescaler
  * @rmtoll   CCFR    REFPSC    FL_CMU_CCL_SetReferenceClockPrescaler
  * @param    prescaler This parameter can be one of the following values:
  *           @arg @ref FL_CMU_CCL_REFCLK_PSC_DIV8
  *           @arg @ref FL_CMU_CCL_REFCLK_PSC_DIV16
  *           @arg @ref FL_CMU_CCL_REFCLK_PSC_DIV32
  *           @arg @ref FL_CMU_CCL_REFCLK_PSC_DIV64
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_CCL_SetReferenceClockPrescaler(uint32_t prescaler)
{
    MODIFY_REG(CMU->CCFR, CMU_CCFR_REFPSC_Msk, prescaler);
}

/**
  * @brief    Get CCL Reference Clock Prescaler
  * @rmtoll   CCFR    REFPSC    FL_CMU_CCL_GetReferenceClockPrescaler
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_CCL_REFCLK_PSC_DIV8
  *           @arg @ref FL_CMU_CCL_REFCLK_PSC_DIV16
  *           @arg @ref FL_CMU_CCL_REFCLK_PSC_DIV32
  *           @arg @ref FL_CMU_CCL_REFCLK_PSC_DIV64
  */
__STATIC_INLINE uint32_t FL_CMU_CCL_GetReferenceClockPrescaler(void)
{
    return (uint32_t)(READ_BIT(CMU->CCFR, CMU_CCFR_REFPSC_Msk));
}

/**
  * @brief    Set CCL Calibration Clock Source
  * @rmtoll   CCFR    CALSEL    FL_CMU_CCL_SetCalibrationClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_CCL_CALCLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_CCL_CALCLK_SOURCE_XTHF
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_CCL_SetCalibrationClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->CCFR, CMU_CCFR_CALSEL_Msk, clock);
}

/**
  * @brief    Get CCL Calibration Clock Source Setting
  * @rmtoll   CCFR    CALSEL    FL_CMU_CCL_GetCalibrationClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_CCL_CALCLK_SOURCE_RCHF
  *           @arg @ref FL_CMU_CCL_CALCLK_SOURCE_XTHF
  */
__STATIC_INLINE uint32_t FL_CMU_CCL_GetCalibrationClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->CCFR, CMU_CCFR_CALSEL_Msk));
}

/**
  * @brief    Set CCL Reference Clock Source
  * @rmtoll   CCFR    REFSEL    FL_CMU_CCL_SetReferenceClockSource
  * @param    clock This parameter can be one of the following values:
  *           @arg @ref FL_CMU_CCL_REFCLK_SOURCE_XTLF
  *           @arg @ref FL_CMU_CCL_REFCLK_SOURCE_RCLP
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_CCL_SetReferenceClockSource(uint32_t clock)
{
    MODIFY_REG(CMU->CCFR, CMU_CCFR_REFSEL_Msk, clock);
}

/**
  * @brief    Get CCL Reference Clock Source Setting
  * @rmtoll   CCFR    REFSEL    FL_CMU_CCL_GetReferenceClockSource
  * @retval   Returned value can be one of the following values:
  *           @arg @ref FL_CMU_CCL_REFCLK_SOURCE_XTLF
  *           @arg @ref FL_CMU_CCL_REFCLK_SOURCE_RCLP
  */
__STATIC_INLINE uint32_t FL_CMU_CCL_GetReferenceClockSource(void)
{
    return (uint32_t)(READ_BIT(CMU->CCFR, CMU_CCFR_REFSEL_Msk));
}

/**
  * @brief    Get CCL Counter value
  * @rmtoll   CCNR    CCNT    FL_CMU_CCL_ReadCounter
  * @retval
  */
__STATIC_INLINE uint32_t FL_CMU_CCL_ReadCounter(void)
{
    return (uint32_t)(READ_BIT(CMU->CCNR, (0xffffU << 0U)) >> 0U);
}

/**
  * @brief    Get CCL interrupt flag
  * @rmtoll   CCISR    CCLIF    FL_CMU_IsActiveFlag_CCLComplete
  * @retval   State of bit (1 or 0).
  */
__STATIC_INLINE uint32_t FL_CMU_IsActiveFlag_CCLComplete(void)
{
    return (uint32_t)(READ_BIT(CMU->CCISR, CMU_CCISR_CCLIF_Msk) == (CMU_CCISR_CCLIF_Msk));
}

/**
  * @brief    Clear CCL interrupt flag
  * @rmtoll   CCISR    CCLIF    FL_CMU_ClearFlag_CCLComplete
  * @retval   None
  */
__STATIC_INLINE void FL_CMU_ClearFlag_CCLComplete(void)
{
    WRITE_REG(CMU->CCISR, CMU_CCISR_CCLIF_Msk);
}

/**
  * @}
  */

/** @defgroup CMU_FL_EF_Init Initialization and de-initialization functions
  * @{
  */

FL_ErrorStatus FL_CMU_SetSystemClock(FL_SystemClock systemClock);

uint32_t FL_CMU_GetPLLClockFreq(void);
uint32_t FL_CMU_GetRCHFClockFreq(void);
uint32_t FL_CMU_GetRCLFClockFreq(void);
uint32_t FL_CMU_GetSystemClockFreq(void);
uint32_t FL_CMU_GetAPBClockFreq(void);
uint32_t FL_CMU_GetAHBClockFreq(void);

/**
  * @}
  */


/**
  * @}
  */

#ifdef __cplusplus
}
#endif

#endif /* __FM33FR0XX_FL_CMU_H*/

/*************************Py_Code_Generator Version: 0.1-0.14-0.1 @ 2021-11-02*************************/
/*************************(C) COPYRIGHT Fudan Microelectronics **** END OF FILE*************************/
